package com.sodius.mdw.log;

import org.eclipse.core.runtime.IStatus;

import com.sodius.mdw.core.util.log.Level;

/**
 * @author Mickael Albert
 *
 */
public class EventStatus implements IStatus {

	private final IStatus[] children;
	private final EventCode code;
	private Throwable exception;
	private EventSeverity severity;
	private final EventContext context;
	
	
	public EventStatus(EventCode code, EventSeverity severity,
			Throwable exception, 
			EventContext context, IStatus... children) {
		if(code==null ||
				severity==null)
			throw new IllegalArgumentException("Code or Severity of an EventStatus cannot be null.");
		this.children = children!=null?children:(new IStatus[]{});
		this.code = code;
		this.exception = exception;
		this.severity = severity;
		this.context=context;
	}

	public IStatus[] getChildren() {
		return children;
	}

	public EventCode getEventCode() {
		return code;
	}

	public Throwable getException() {
		return exception;
	}

	
	public String getMessage() {
		return code.getDescription();
	}

	public int getSeverity() {
		return severity.getSeverity();
	}
	
	public EventSeverity getEventSeverity() {
		return severity;
	}
	
	/**
	 * @return The level of error in a MDWorkench context
	 */
	public Level getLevel() {
		return severity.getLevel();
	}

	public EventContext getContext() {
		return context;
	}

	public boolean isOK() {
		return matches(OK);
	}	
	
	public boolean matches(int severityMask) {
		return getSeverity()==severityMask;
	}

	public int getCode() {
		return code.getCode();
	}

	public String getPlugin() {
		// TODO Should define a real plugin
		return "com.sodius.mdw.log";
	}

	public boolean isMultiStatus() {
		return children!=null && children.length>0;
	}
	
	public static EventStatus createStatus(EventCode code, EventSeverity severity,
			Throwable exception, 
			EventContext context, IStatus... children)
	{
		return new EventStatus(code, severity, exception, context, children);
	}
	
	public void setException(Throwable exception) {
		this.exception = exception;
	}

	public void setSeverity(EventSeverity severity) {
		this.severity = severity;
	}

	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		appendRepresentation(builder);
		return builder.toString();
	}
	
	/**
	 * Create the representation of this EventStatus into builder
	 * 
	 * @param builder
	 */
	protected void appendRepresentation(StringBuilder builder)
	{
		builder.append(code.getDescription());
		if(context!=null)
		{
			builder.append(" at ");
			builder.append(context.toString());
		}
	}
	
	
	
	
}
