package com.sodius.rhapsody.toolkit;

import java.io.File;
import java.util.Arrays;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import com.sodius.mdw.core.util.progress.NullProgressMonitor;
import com.sodius.rhapsody.toolkit.engine.EngineContext;
import com.sodius.rhapsody.toolkit.engine.HeadlessEngineContext;
import com.sodius.rhapsody.toolkit.engine.XMIEngine;
import com.sodius.rhapsody.toolkit.properties.RhapsodyXMIToolkitParameters;
import com.sodius.rhapsody.toolkit.ui.RhapsodySafeRunnable;
import com.sodius.rhapsody.toolkit.ui.RhapsodyXMIToolkit;
import com.sodius.rhapsody.toolkit.ui.ToolkitUIManager;
import com.sodius.rhapsody.toolkit.util.thread.RhapsodyAPIExec;
import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.RhapsodyAppServer;

public class XMIToolkitApplication {// implements IPlatformRunnable {

	/*
	 * public Object run(Object args) throws Exception { main((String[]) args);
	 * return IPlatformRunnable.EXIT_OK; }
	 */

	public void run(String[] args) {
		XMIToolkitApplication.main(args);
	}

	public static void mainPlugin(String[] args, IRPApplication irpApplication) {
		// look for silent mode
		List<String> argList = Arrays.asList(args);

		if (argList.contains("-silent") || argList.contains("silent")) {
			// do silent mode
			runHeadless(args, irpApplication);
		} else {
			runJFaceUI(args, irpApplication);
		}
	}
	
	public static void main(String[] args) {
		// look for silent mode
		List<String> argList = Arrays.asList(args);

		if (argList.contains("-silent") || argList.contains("silent")) {
			// do silent mode
			runHeadless(args, null);
		} else {
			runJFaceUI(args, null);
		}
	}

	private static void runHeadless(String[] args, IRPApplication irpApplication) {
		try {
			EngineContext context = new HeadlessEngineContext(
					readParameters(args));
			XMIEngine engine = new XMIEngine(context, irpApplication);
			engine.run(new NullProgressMonitor());
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	private static RhapsodyXMIToolkitParameters readParameters(String[] args) {
		RhapsodyXMIToolkitParameters params = new RhapsodyXMIToolkitParameters();
		try {
			params.read(args);
		} catch (Throwable t) {
			System.out.println("Unable to read properties.");
		}

		return params;
	}

	private static void runJFaceUI(String[] args, IRPApplication irpApplication) {
		List<String> argList = Arrays.asList(args);
		boolean isPlugin = argList.contains("-asPlugin");
		final boolean isImport = argList.contains("IMPORT");
		try {
			RhapsodyXMIToolkit toolkit = null;
			if (isPlugin) {
				String rhpValue = (String) argList.get(argList
						.indexOf("-asPlugin") + 1);
				int rhpID = 0;
				try {
					rhpID = Integer.parseInt(rhpValue);
				} catch (NumberFormatException e) {
					// do nothing
				}
				toolkit = new RhapsodyXMIToolkit(args, rhpID);
			}else if(irpApplication!=null)
			{
				isPlugin=true;
				toolkit = new RhapsodyXMIToolkit(args, irpApplication);
			} else {
				toolkit = new RhapsodyXMIToolkit(args);
			}
			if(argList.contains("-debug"))
			{
				final File rhpLog= (new File(toolkit.getToolkitDirectory(),"rhpLog.txt"));
				RhapsodyAPIExec.synchExec(new Runnable() {
					public void run() {
						//toolkit.getOutLogger().info("Setting debug mode. RhpLog in " + rhpLog.getAbsolutePath());
						RhapsodyAppServer.setLogFile(rhpLog.getAbsolutePath());
					}
				});
			}
			
			ToolkitUIManager manager = new ToolkitUIManager(toolkit);
			if (isImport) {
				toolkit.getParameters().setImportMode();
				manager.openImportWizard();
			} else {
				toolkit.getParameters().setXMIExportMode();
				manager.openExportWizard();
			}

		} catch (Throwable e) {
		    displayErrorUI(Display.getDefault().getActiveShell(),e);
			Display.getDefault().dispose();
			if (!isPlugin)
				System.exit(1);
		}
	}
	
	public static void displayErrorUI(Shell shell, Throwable e)
	{
		e.printStackTrace();
		RhapsodySafeRunnable.handleException(shell, e);
	}
}
