package com.sodius.rhapsody.toolkit.engine.uml21;

import java.io.File;
import java.io.IOException;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.eval.launch.LaunchConfiguration;
import com.sodius.mdw.core.eval.project.Project;
import com.sodius.mdw.core.util.io.IOUtils;
import com.sodius.mdw.core.util.progress.ProgressMonitor;
import com.sodius.mdw.core.util.progress.SubProgressMonitor;
import com.sodius.mdw.corext.project.ProjectClasspath;
import com.sodius.mdw.corext.project.ProjectClasspathFactory;
import com.sodius.rhapsody.toolkit.engine.EngineContext;
import com.sodius.rhapsody.toolkit.engine.EngineRunner;

abstract class UML21EngineRunner implements EngineRunner {

	protected EngineContext context;
	protected String toolkitDirectory;
    
    public UML21EngineRunner(EngineContext context, String toolkitDirectory) {
    	this.context = context;
    	this.toolkitDirectory = toolkitDirectory;
    }
    
    protected final EngineContext getContext() {
		return context;
	}
    
    void evaluate(LaunchConfiguration configuration, ProgressMonitor monitor) throws CoreException {
        // evaluate
        context.getParameters().getMDWorkbench().getEvaluationManager().evaluate(
        		configuration, 
        		new SubProgressMonitor(monitor, 10));
    }
    
    protected Project getProject(String classPathFileName) throws CoreException {
        File classpathFile = null;
        try {
            // create classpath
            classpathFile = File.createTempFile(classPathFileName, ".classpath");
            final ProjectClasspath classpath = createProjectClasspath("XMIToolkitForRhapsody");
            classpath.save(classpathFile);

            // load project
            Project project = context.getParameters().getMDWorkbench().getEvaluationManager().loadProject(classpathFile);
            return project;
        } catch (final IOException e) {
            throw new CoreException(e);
        } finally {
            // remove classpath
            IOUtils.deleteFile(classpathFile);
        }  
    }
    
    private ProjectClasspath createProjectClasspath(final String projectName) {
        final ProjectClasspath classpath = ProjectClasspathFactory.createProjectClasspath(projectName);
        // add entries
        // Rhapsody2UML21.jar or UML21toRhapsody.jar (full name with version)
        classpath.addEntry(getTransformationPluginPath().getAbsolutePath(), true);
        // rhapsodyUmlShared.jar
        classpath.addEntry(getRhapsodyUMLSharedPluginPath().getAbsolutePath(), true);
        return classpath;
    }
    
    protected File getMDWorkbenchFolder(){
    	return new File(this.toolkitDirectory + File.separator + "lib" + File.separator + "mdworkbench");
    }
    
    protected File getRhapsodyUMLSharedPluginPath() {
    	return new File(getMDWorkbenchFolder(), getRhapsodyPlugin(getMDWorkbenchFolder(), "com.sodius.mdw.rhapsodyUmlShared_"));
	}
    
    protected abstract File getTransformationPluginPath();
    
    protected String getRhapsodyPlugin(File folder, String name){
    	for (File fileEntry : folder.listFiles()) {
			if(fileEntry.getName().startsWith(name)){
				return fileEntry.getName();
			}
		}
    	return null;
    }
}
