package com.sodius.rhapsody.toolkit.engine.uml21;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.eval.Evaluable;
import com.sodius.mdw.core.eval.EvaluationContext;
import com.sodius.mdw.core.eval.project.Project;
import com.sodius.mdw.core.model.Model;
import com.sodius.mdw.core.model.io.ConnectorDescriptor;
import com.sodius.mdw.core.util.progress.ProgressMonitor;
import com.sodius.rhapsody.toolkit.engine.EngineContext;
import com.sodius.rhapsody.toolkit.util.RhapsodyUnitHandler;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.IRPProject;

public class UML21ExportRunner extends UML21EngineRunner {

    public static String rhapsodyCOMConnectorSwitcherKey = "rhapsody.com.connector.switcher";
    public static String rhapsodyEmbeddedConnectorSwitcherKey = "rhapsody.embedded.connector.switcher";

    public UML21ExportRunner(EngineContext context, String toolkitDirectory) {
        super(context, toolkitDirectory);
    }

    @Override
    protected File getRhapsodyUMLSharedPluginPath() {
        return new File(getMDWorkbenchFolder(), getRhapsodyPlugin(getMDWorkbenchFolder(), "com.sodius.mdw.rhapsodyUmlShared_"));
    }

    @Override
    protected File getTransformationPluginPath() {
        return new File(getMDWorkbenchFolder(), getRhapsodyPlugin(getMDWorkbenchFolder(), "com.sodius.mdw.Rhapsody2UML21_"));
    }

    public void run(ProgressMonitor monitor) throws CoreException {
        String RhapsodyConnectorID = "Rhapsody On-Demand";
        if (Boolean.getBoolean(rhapsodyCOMConnectorSwitcherKey)) {
            RhapsodyConnectorID = "Rhapsody Application";
        }
        if (Boolean.getBoolean(rhapsodyEmbeddedConnectorSwitcherKey)) {
            RhapsodyConnectorID = "Rhapsody Embedded On-Demand";
        }
        ConnectorDescriptor rodDesc = context.getParameters().getMDWorkbench().getMetamodelManager().getMetamodel("rhapsody")
                .getModelReaderDescriptor(RhapsodyConnectorID);

        rodDesc.setProperty("activeApplication", context.getXMIEngine().getApplication());

        final boolean isRhapsodyCL = context.getParameters().isRhapsodyCL();
        if (isRhapsodyCL) {
            rodDesc.setProperty("exportDiagrams", false);
        }

        final Model umlModel = context.getParameters().getMDWorkbench().getMetamodelManager().getMetamodel("uml21").createModel();
        final Model rhaModel = context.getParameters().getMDWorkbench().getMetamodelManager().getMetamodel("rhapsody").readModel(RhapsodyConnectorID,
                new File(toolkitDirectory, "Properties.ini").getAbsolutePath());
        final String namespaces = getUnitsSelection(getContext().getParameters().getSelectedUnitsToExport());
        Project project = getProject("rhapsody2UML");
        context.getParameters().getMDWorkbench().getConfiguration().setLogger(new EventToPrintLogger(context.getOutLogger(), context.getErrLogger()));
        final String exportRHPExtension = "exportRHPExtension=" + Boolean.toString(context.getParameters().getExportRhapsodyExtension());
        context.getParameters().getMDWorkbench().getEvaluationManager().evaluate(project, new Evaluable() {

            public void evaluate(EvaluationContext context) throws CoreException {
                List<Object> ruleSetArguments = new ArrayList<Object>();
                // Adding Rhapsody Model
                ruleSetArguments.add(rhaModel);
                // Adding UMLModel
                ruleSetArguments.add(umlModel);
                // Adding setup properties
                ruleSetArguments.add(exportRHPExtension);

                List<Object> ruleArguments = new ArrayList<Object>();
                // adding namespaces
                ruleArguments.add(namespaces);
                context.evaluateRule("com.sodius.mdw.Rhapsody2UML21.rulesets.frameworkRuleset", ruleSetArguments, "run", ruleArguments);

            }

        }, monitor);
        // save file

        int count = UnitSplitter.split(rhaModel, umlModel, getContext().getParameters().getXMIFilename(), namespaces,
                getContext().getParameters().getExportInSeparateFiles(), context.getParameters().getFormat().umlVersion,
                getContext().getParameters().getExportEmbeddedSysMLProfile(), context.getOutLogger()); // @See RTC 138401

        context.getDirectOutLogger().info(count + " exported elements.\nComplete logs at " + context.getParameters().getLogDirectory());

    }

    private String getUnitsSelection(List<Object> selectedUnits) {
        String unitsSelection = "";
        for (Object unitObjectItem : selectedUnits) {
            if (unitObjectItem instanceof RhapsodyUnitHandler) {
                IRPModelElement unitObject = ((RhapsodyUnitHandler) unitObjectItem).getIRPInterface();
                if (unitObject instanceof IRPProject) {
                    return "";// all are selected
                }

                if (unitsSelection.length() > 0) {
                    unitsSelection += ",";
                }
                unitsSelection += unitObject.getGUID();
            }
        }
        return unitsSelection;
    }

}
