package com.sodius.rhapsody.toolkit.engine.uml21;

import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.DynamicEObjectImpl;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.xmi.XMLResource;

import com.sodius.mdw.core.model.MDWObject;
import com.sodius.mdw.core.model.Model;
import com.sodius.mdw.core.util.log.Logger;
import com.sodius.mdw.metamodel.rhapsody.Profile;
import com.sodius.mdw.metamodel.rhapsody.Project;
import com.sodius.mdw.metamodel.rhapsody.Property;
import com.sodius.mdw.metamodel.rhapsody.Unit;
import com.sodius.mdw.metamodel.uml21.Element;
import com.sodius.mdw.metamodel.uml21.Package;
import com.sodius.mdw.metamodel.uml21.Stereotype;
import com.sodius.mdw.metamodel.uml21.UMLPackage;
import com.sodius.mdw.metamodel.uml21.resource.OMGUMLVersionRegistry;
import com.sodius.mdw.metamodel.uml21.resource.ProfileRegistry;
import com.sodius.mdw.metamodel.uml21.resource.UML22UMLResource;
import com.sodius.mdw.metamodel.uml21.resource.UMLResource;
import com.sodius.mdw.metamodel.uml21.resource.XMI2UMLResource;
import com.sodius.mdw.metamodel.uml21.resource.XMI2UMLVersionRegistry;
import com.sodius.mdw.metamodel.uml21.xmi.UML22UMLResourceFactoryImplExt;
import com.sodius.mdw.metamodel.uml21.xmi.UMLResourceFactoryImplExt;
import com.sodius.mdw.metamodel.uml21.xmi.XMI2UMLResourceFactoryImplExt;

public class UnitSplitter {

	
	private static XMI2UMLVersionRegistry getXMIVersion(OMGUMLVersionRegistry umlVersion)
	{
		for (XMI2UMLVersionRegistry xmiVersion : XMI2UMLVersionRegistry.values()) {
			if(xmiVersion.getOmgEquivalentVersion()==umlVersion)
				return xmiVersion;
		}
		return XMI2UMLResource.DEFAULT_UML_VERSION;
	}
	
	public static int split(Model rhaModel, Model umlModel, String path, String namespaces, boolean asSeparateUnits, OMGUMLVersionRegistry umlVersion, boolean embedProfiles, Logger log) //@See RTC 138401
	{
		int totalElementWriten = 0;
		List<String> selection = Arrays.asList(namespaces.split("\\,"));
		if(namespaces.length()==0)
			selection=new ArrayList<String>();
		Set<String> hashSelection = new HashSet<String>(selection);
		File folder = new File(path);
		File fullFile = null;
		//TODO : check all rights
		if(!asSeparateUnits)
		{
			fullFile = folder;
			folder = folder.getParentFile();
		}
		folder.mkdirs();
		
		//1) get resourceSet
		com.sodius.mdw.metamodel.uml21.Model umlStandardModel = (com.sodius.mdw.metamodel.uml21.Model)umlModel.getInstances("Model").detect("name", "uml");
		if(umlStandardModel!=null)
		{
			ResourceSet resourceSet=umlStandardModel.eResource().getResourceSet();
			resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(XMI2UMLResource.FILE_EXTENSION, new XMI2UMLResourceFactoryImplExt(getXMIVersion(umlVersion))); //@See Mantis 0003118 @See RTC 138401
			resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(UML22UMLResource.FILE_EXTENSION, new UML22UMLResourceFactoryImplExt());
			resourceSet.getPackageRegistry().put(UMLPackage.eNS_URI,UMLPackage.eINSTANCE);
			resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(UMLResource.FILE_EXTENSION, new UMLResourceFactoryImplExt());
			//2) create a resource for each unit
			Map<String, Resource> mapUnitResource = new HashMap<String, Resource>();
			//Map<String, Unit> mapGUIDUnit = new HashMap<String, Unit>();

			
			Resource projectResource=null;
			if(!asSeparateUnits)
				projectResource=getResource(resourceSet, folder, fullFile.getName(),umlVersion);
			
			//tag owners of selected units
			Set<String> ownersGUID = new HashSet<String>();
			for(Unit separateUnit : rhaModel.<Unit>getInstances("Unit", true).select("isSeparateSaveUnit", true))
			{
				if(hashSelection.contains(separateUnit.getGUID()))
				{
					registerOwners(hashSelection, ownersGUID, separateUnit);
				}
			}
			
			
			for(Unit separateUnit : rhaModel.<Unit>getInstances("Unit", true).select("isSeparateSaveUnit", true))
			{
				Resource resource;
				if(asSeparateUnits // in separateUnit mode : each unit is a separate file
						|| (!hashSelection.isEmpty() && !hashSelection.contains(separateUnit.getGUID()) && !ownersGUID.contains(separateUnit.getGUID()))) //in non separate unit mode, all selected units are in same file, other considered in there respective resource (except for owners of selection)
				{
					resource = getResource(resourceSet, folder, getFilename(separateUnit),umlVersion);
					mapUnitResource.put(separateUnit.getGUID(), resource);
				}
//				else
//				{
//					resource = projectResource; // in non separate unit, selected units are in the fullfile
//				}
			}
			
			
			//Add project into this list
			for (Project project : rhaModel.<Project>getInstances("Project")) {
				if(!mapUnitResource.keySet().contains(project.getGUID()))
				{
					
					Resource resource;
					if(asSeparateUnits) // in separateUnit mode : each unit is a separate file
							
					{
						resource = getResource(resourceSet, folder, getFilename(project),umlVersion);
					}
					else
					{
						resource = projectResource; // in non separate unit, selected units are in the fullfile
					}
					
					if(asSeparateUnits)
						projectResource=resource;
					
					mapUnitResource.put(project.getGUID(), resource);
				}
			}
			
			//Manage SysMLProfile
			if(embedProfiles)
			{
				Profile sysMLProfile = rhaModel.<Profile>getInstances("Profile").detect("name", "SysML");
				if(sysMLProfile!=null)
				{
					//Search resource
					Resource sysMLResource = null;
					for (Resource subResource : resourceSet.getResources()) {
						String uri = subResource.getURI().toString();
						if(ProfileRegistry.INTERNAL_SYSML.getAllURIs().contains(uri) ||
								ProfileRegistry.INTERNAL_SYSML_1_4.getAllURIs().contains(uri))
						{
							sysMLResource=subResource;
							break;
						}
					}
					if(sysMLResource!=null)
					{
						Resource newResource = null;
						if(asSeparateUnits) // in separateUnit mode : each unit is a separate file
							
						{
							newResource = getResource(resourceSet, folder,"SysML.profile.xmi", umlVersion);
							mapUnitResource.put(sysMLProfile.getGUID(), newResource);
						}
						else
						{
							newResource = projectResource; // in non separate unit, selected units are in the fullfile
						}
						if(sysMLResource.getEObject("_0")==null)
							newResource.getContents().add(sysMLResource.getEObject("SysML"));
						else 
						newResource.getContents().add(sysMLResource.getEObject("_0"));
					}
				}
			}
			
			
			//3) search unit in uml model and affect them in each ressources
			Stereotype stRhpElement = umlModel.<Stereotype>getInstances("Stereotype").detect("name", "RhpModelElement");
			for (MDWObject object : umlModel.getInstances()) {
				if(object instanceof Element)
				{
					Element element = (Element)object;
					if(((InternalEObject)element).eDirectResource()==null)
					{
						String guid = element.eUniqueID();
						if(guid==null && stRhpElement!=null && element.isStereotypeApplied(stRhpElement))
						{
							if(element.hasValue(stRhpElement, "guid"))
								guid=(String)element.getValue(stRhpElement, "guid");
						}
						if(guid!=null)
						{
							if(mapUnitResource.keySet().contains(guid))
							{
								Resource resource = mapUnitResource.get(guid);
								resource.getContents().add(element);
							}
						}
					}
				}
			}
				
			//4) find solution for elements without resources : Profiles
			Map<Resource, List<EObject>> elementsToAddInTail = new HashMap<Resource, List<EObject>>();
			for (MDWObject object : umlModel.getInstances()) {
				if(object instanceof Element)
				{
					Element element = (Element)object;
					if(element.getOwner()==null)
					{
						if(element.eResource()==null)
						{
							projectResource.getContents().add(element);
						}
						//setupElementOnResource(element.eResource(), element, elementsToAddInTail);
					}
				}
			}
			
			Map<Resource, int[]> counter = new HashMap<Resource, int[]>();
			for (MDWObject object : umlModel.getInstances()) {
				if(object instanceof Element)
				{
					Element element = (Element)object;
					if(element.eResource()!=null && element.eResource().getURI().isFile()) //should always be the case
					{
						incrementResourceContentCount(counter, element.eResource());
						setupElementOnResource(element.eResource(), element, elementsToAddInTail);
					}
				}
			}
			
			for (Resource resource : elementsToAddInTail.keySet()) {
				resource.getContents().addAll(elementsToAddInTail.get(resource));
			}
			
			//5) go through each selected units, and save them
			
			if(namespaces.equals(""))
			{
				for (Resource resource : resourceSet.getResources()) {
				{
					totalElementWriten+=saveResource(resource,log,counter);
				}
				}
			}
			else
			{
				if(asSeparateUnits)
				{
					Set<Resource> resourceSaved = new HashSet<Resource>();
					for (String guid : selection) {
						Resource resource = mapUnitResource.get(guid);
						totalElementWriten+=saveResource(resource,log,counter);
						resourceSaved.add(resource);
					}
					for (String guid : ownersGUID) {
						Resource resource = mapUnitResource.get(guid);
						if(resource!=null && !resourceSaved.contains(resource))
						{
							totalElementWriten+=saveResource(resource,log,counter);
							resourceSaved.add(resource);
						}
						
					}
				}else
					totalElementWriten+=saveResource(projectResource,log,counter);
			}
			
		}
		else
		{
			System.out.println("UML model not found");
		}
		
		return totalElementWriten;
	}
	
	private static void incrementResourceContentCount(Map<Resource, int[]> counter, Resource resource)
	{
		int[] count = counter.get(resource);
		if(count==null)
		{
			count = new int[]{0};
			counter.put(resource, count);
		}
		count[0]++;
	}
	
	private static void registerOwners(Set<String> hashSelection, Set<String> ownersGUID, Unit unit)
	{
		if((unit.getOwner() instanceof Unit) && !hashSelection.contains(unit.getOwner().getGUID()))
		{
			ownersGUID.add(unit.getOwner().getGUID());
			registerOwners(hashSelection, ownersGUID, (Unit)unit.getOwner());
		}
	}
	
	private static String getFilename(Unit unit)
	{
		Property xmiIDProperty = (Property)unit.getProperties().detect("key","XMI.Class.id");
		if(xmiIDProperty!=null)
		{
			return xmiIDProperty.getValue().split("#")[0];
		}
		return unit.getFilename()+".xmi";
	}
	
	
	private static Resource getResource(ResourceSet resourceSet, File folder, String resourceName, OMGUMLVersionRegistry umlVersion)
	{
		for (Resource resource : resourceSet.getResources()) {
			if(resource.getURI().isFile() && new File(resource.getURI().toFileString()).getName().equals(resourceName))
				return resource;
		}
		
		Resource r = resourceSet.createResource(URI.createFileURI(new File(folder, resourceName).getAbsolutePath()));
		if(r==null)
		{
			r= new XMI2UMLResourceFactoryImplExt(getXMIVersion(umlVersion)).createResource(URI.createFileURI(new File(folder, resourceName).getAbsolutePath()));
			resourceSet.getResources().add(r);
		}
		return r;
		
	}
	
	private static int saveResource(Resource resource, Logger log, Map<Resource, int[]> counter)
	{
		//@see 167987
		if(!resource.getContents().isEmpty() && resource.getURI().isFile())
		{
			try {
				resource.save(new HashMap<Object, Object>());
				int[] nbElements = counter.get(resource);
				if(nbElements!=null)
					return nbElements[0];
			} catch (IOException e) {
				log.error("Error when saving ressource " + resource.getURI().toFileString(), e);
			}
		}
		return 0;
	}
	
	private static void setupElementOnResource(Resource resource, EObject object, Map<Resource, List<EObject>> elementsToAddInTail)
	{
		//Manage stereotype application
		if(object instanceof Element && !((Element)object).getStereotypeApplications().isEmpty())
		{
			for (EObject appl : ((Element)object).getStereotypeApplications()) {
				if(appl instanceof DynamicEObjectImpl && ((DynamicEObjectImpl)appl).eDirectResource()==null)
				{
					List<EObject> listElements;
					if(elementsToAddInTail.containsKey(resource))
						listElements=elementsToAddInTail.get(resource);
					else
						listElements=new ArrayList<EObject>();
					listElements.add(appl);
					elementsToAddInTail.put(resource, listElements);
				}
			}
		}
		
		//manage profile application
		try{
		if(object instanceof Package && !(object instanceof com.sodius.mdw.metamodel.uml21.Model) && ((InternalEObject)object).eDirectResource()!=null){
			//if a unit package, we create additional profile application on it for reload.
			Package pack = (Package)object;
			List<com.sodius.mdw.metamodel.uml21.Profile> allAppliedProfiles = new ArrayList<com.sodius.mdw.metamodel.uml21.Profile>(pack.getAllAppliedProfiles());
			allAppliedProfiles.removeAll(pack.getAppliedProfiles());
			for (com.sodius.mdw.metamodel.uml21.Profile profile : allAppliedProfiles) {
				pack.applyProfile(profile);
			}
		}
		}catch(Exception e){
			// avoid crash
		}
		//manage XMIIDs
		if(resource.getURI().isFile()) //TODO : not well in case of SysML
		{
			if(object instanceof Element)
			{
				String guid =getGUID((Element)object); 
				if(guid!=null)
					try {
						((XMLResource)resource).setID(object, URLEncoder.encode(guid, "UTF-8"));
					} catch (UnsupportedEncodingException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
			}
		}
	}
	
	private static String getGUID(Element element)
	{
		Stereotype st = (Stereotype) element.getAppliedStereotypes().detect("name", "RhpModelElement");
		if(st!=null)
		{
			if(element.hasValue(st, "lastKnownURI"))
			{
				String xmiid = (String)element.getValue(st, "lastKnownURI");
				if(xmiid.length()>0)
					return xmiid.split("#")[1];;
			}
			if(element.hasValue(st, "guid"))
			{
				String xmiid = (String)element.getValue(st, "guid");
				if(xmiid.length()>0)
					return xmiid;
			}
		}
		return element.eUniqueID();
	}
	
	
	
	
}
