package com.sodius.rhapsody.toolkit.properties;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.xml.sax.SAXException;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.MDWorkbench;
import com.sodius.mdw.core.MDWorkbenchFactory;
import com.sodius.mdw.core.model.Metamodel;
import com.sodius.mdw.core.model.MetamodelManager;
import com.sodius.mdw.core.model.Model;
import com.sodius.mdw.core.model.io.ConnectorDescriptor;
import com.sodius.rhapsody.toolkit.ui.ImportMethodControl;

public class RhapsodyXMIToolkitParameters {
    protected static final String KEY_RELEASE = "toolkit.release";
    protected static final String KEY_TOOLKIT_DIRECTORY = "toolkit.directory";
    protected static final String KEY_SILENT_MODE = "silent";
    protected static final String KEY_DEBUG_MODE = "debug";

    private static final String KEY_RHAPSODY_PROPERTIES_FILE = "propertiesFile";
    public static final String DEFAULT_RHAPSODY_PROPERTIES_FILE = "Properties.ini";
    public static String RHAPSODY_PROPERTIES_FILE = DEFAULT_RHAPSODY_PROPERTIES_FILE;

    public static final String EXTENSION_DIAGRAM_FILE = "di";

    private static final String KEY_RHAPSODY_TAGS_FILE = "tagsFile";
    public static final String DEFAULT_RHAPSODY_TAGS_FILE = "Tags.ini";
    public static String RHAPSODY_TAGS_FILE = DEFAULT_RHAPSODY_TAGS_FILE;

    public static final String KEY_SILENT_FILENAME = "silentFile";

    public static final String KEY_IMPORT_COLLABORATIONS = "importCollaborations";
    public static final String KEY_IMPORT_DEPLOYMENTS = "importDeployments";
    public static final String KEY_IMPORT_STATEMACHINES = "importStatemachines";
    public static final String KEY_IMPORT_SUBSTATES_AS_NESTEDCHART = "importSubstatesAsNestedChart";

    private static final String KEY_IMPORT_COMPOSITIONS_AS_PARTS = "importCompositionsAsParts";
    private static final String KEY_IMPORT_STATECHART = "importStatechart";
    private static final String KEY_IMPORT_SEQUENCE_DIAGRAM = "importSequenceDiagram";
    private static final String KEY_IMPORT_CLASS_DIAGRAM = "importClassDiagram";
    private static final String KEY_IMPORT_USECASE_DIAGRAM = "importUseCaseDiagram";
    private static final String KEY_IMPORT_COMPONENT_DIAGRAM = "importComponentDiagram";
    private static final String KEY_IMPORT_DEPLOYMENT_DIAGRAM = "importDeploymentDiagram";
    private static final String KEY_IMPORT_COLLABORATION_DIAGRAM = "importCollaborationDiagram";
    private static final String KEY_IMPORT_STRUCTURE_DIAGRAM = "importStructureDiagram";
    private static final String KEY_IMPORT_ACTIVITY_DIAGRAM = "importActivityDiagram";

    public static final String KEY_EXPORT_METHODS = "exportMethods";
    public static final String KEY_EXPORT_TAGS = "exportTags";
    public static final String KEY_EXPORT_STATEMACHINES = "exportStatemachines";
    public static final String KEY_EXPORT_ANNOTATIONS = "exportAnnotations";
    public static final String KEY_EXPORT_COLLABORATIONS = "exportCollaborations";
    public static final String KEY_EXPORT_DESCRIPTION_HTML = "exportDescriptionHTML";
    public static final String KEY_EXPORT_DEPLOYMENTS = "exportDeployments";
    public static final String KEY_EXPORT_PROPERTIES = "exportProperties";
    public static final String KEY_EXPORT_DESCRIPTION_RTF = "exportDescriptionRTF";

    private static final String KEY_EXPORT_PACKAGE_LINKS = "riaExportPackageLinks";

    private static final String KEY_EXPORT_DIAGRAMS_ANNOTATIONS = "exportDiagramsAnnotations";
    private static final String KEY_EXPORT_COMMENTS = "exportComments";
    private static final String KEY_EXPORT_CONSTRAINTS = "exportConstraints";
    private static final String KEY_EXPORT_REQUIREMENTS = "exportRequirements";

    private static final String KEY_EXPORT_PARTIAL_ID = "exportPartial";

    public static final String KEY_EXPORT_WITH_RHAPSODY_EXTENSION = "exportRHPExtension";
    private static final String KEY_AS_PLUGIN = "asPlugin";
    private static final String KEY_CREATE_CLASS_FOR_SIGNAL = "createClassForSignal";
    private static final String KEY_ADD_ROLE_NAMES = "addRoleNames";
    private static final String KEY_USE_TAU_COMPATIBLE_MESSAGES = "useTauCompatibleMessages";
    private static final String KEY_REALIZE_ALL_MESSAGES = "realizeAllMessages";
    private static final String KEY_DONT_CREATE_OBJECTS = "dontCreateObjects";
    private static final String KEY_ADD_NEW_LINE_FOR_ACTION = "addNewLineForAction";
    private static final String KEY_CREATE_CLASSIFIERS_FOR_UNREALIZED_CLASSIFIER_ROLE = "createClassifiersForUnrealizedClassifierRoles";

    public static final String KEY_EXPORT_EMBEDDED_SYSML_PROFILE = "embeddedSysMLProfile";
    public static final boolean DEFAULT_EXPORT_EMBEDDED_SYSML_PROFILE = true;
    public static final boolean DEFAULT_EXPORT_WITH_RHAPSODY_EXTENSION = true;

    public static final boolean DEFAULT_IMPORT_STATEMACHINES = true;
    public static final boolean DEFAULT_IMPORT_SUBSTATES_AS_NESTEDCHART = false;
    public static final boolean DEFAULT_IMPORT_COLLABORATIONS = true;
    public static final boolean DEFAULT_IMPORT_DEPLOYMENTS = true;

    private static final boolean DEFAULT_IMPORT_ACTIVITY_DIAGRAM = false;

    private static final boolean DEFAULT_IMPORT_COMPOSITION_AS_PARTS = false;
    private static final boolean DEFAULT_IMPORT_STATECHART = false;
    private static final boolean DEFAULT_IMPORT_SEQUENCE_DIAGRAM = false;
    private static final boolean DEFAULT_IMPORT_CLASS_DIAGRAM = false;
    private static final boolean DEFAULT_IMPORT_USECASE_DIAGRAM = false;
    private static final boolean DEFAULT_IMPORT_COMPONENT_DIAGRAM = false;
    private static final boolean DEFAULT_IMPORT_DEPLOYMENT_DIAGRAM = false;
    private static final boolean DEFAULT_IMPORT_COLLABORATION_DIAGRAM = false;
    private static final boolean DEFAULT_IMPORT_STRUCTURE_DIAGRAM = false;

    public static final boolean DEFAULT_EXPORT_METHODS = false;
    public static final boolean DEFAULT_EXPORT_STATEMACHINES = false;
    public static final boolean DEFAULT_EXPORT_COLLABORATIONS = false;
    public static final boolean DEFAULT_EXPORT_DEPLOYMENTS = false;
    public static final boolean DEFAULT_EXPORT_PROPERTIES = false;
    public static final boolean DEFAULT_EXPORT_TAGS = false;
    public static final boolean DEFAULT_EXPORT_DESCRIPTION_HTML = false;
    public static final boolean DEFAULT_EXPORT_DESCRIPTION_RTF = false;
    public static final boolean DEFAULT_EXPORT_ANNOTATIONS = true;

    private static final boolean DEFAULT_EXPORT_PACKAGE_LINKS = false;

    private static final boolean DEFAULT_EXPORT_DIAGRAMS_ANNOTATIONS = false;
    private static final boolean DEFAULT_EXPORT_COMMENTS = true;
    private static final boolean DEFAULT_EXPORT_CONSTRAINTS = true;
    private static final boolean DEFAULT_EXPORT_REQUIREMENTS = true;

    private static final int DEFAULT_EXPORT_PARTIAL_ID = 0;

    private static final boolean DEFAULT_CREATE_CLASS_FOR_SIGNAL = false;

    private static final boolean DEFAULT_ADD_ROLE_NAMES = false;

    private static final boolean DEFAULT_USE_TAU_COMPATIBLE_MESSAGES = false;

    private static final boolean DEFAULT_REALIZE_ALL_MESSAGES = false;

    private static final boolean DEFAULT_DONT_CREATE_OBJECTS = false;

    private static final boolean DEFAULT_ADD_NEW_LINE_FOR_ACTION = false;

    private static final boolean DEFAULT_CREATE_CLASSIFIERS_FOR_UNREALIZED_CLASSIFIER_ROLE = false;

    // Unisys extensions properties
    private static final String KEY_TOOL_NAME = "ToolName";
    private static final String KEY_ASSOCIATION_STYLE = "AssociationStyle";
    private static final String KEY_HORIZONTAL_SYNCH_STYLE = "HorizontalSynchStyle";
    private static final String KEY_NOTE_STYLE = "NoteStyle";
    private static final String KEY_VERTICAL_SYNCH_STYLE = "VerticalSynchStyle";
    private static final String KEY_X_SCALING_FACTOR = "XScalingFactor";
    private static final String KEY_Y_SCALING_FACTOR = "YScalingFactor";
    private static final String KEY_WIDTH_SCALING_FACTOR = "WidthScalingFactor";
    private static final String KEY_HEIGHT_SCALING_FACTOR = "HeightScalingFactor";
    private static final String KEY_DEFAULT_WIDTH = "DefaultWidth";
    private static final String KEY_DEFAULT_HEIGHT = "DefaultHeight";

    private static final String DEFAULT_TOOL_NAME = "Rational Rose 98";
    private static final String DEFAULT_ASSOCIATION_STYLE = "Association:LineColor.Blue= 51,LineColor.Green= 0,LineColor.Red= 153,";
    private static final String DEFAULT_HORIZONTAL_SYNCH_STYLE = "SyncItem, Horizontal=1,";
    private static final String DEFAULT_NOTE_STYLE = "Note:Font.Blue= 0,Font.Green= 0,Font.Red= 0,FillColor.Transparent=1,";
    private static final String DEFAULT_VERTICAL_SYNCH_STYLE = "SyncItem, Horizontal=0,";
    private static final int DEFAULT_X_SCALING_FACTOR = 1;
    private static final int DEFAULT_Y_SCALING_FACTOR = 1;
    private static final int DEFAULT_WIDTH_SCALING_FACTOR = 1;
    private static final int DEFAULT_HEIGHT_SCALING_FACTOR = 1;
    private static final int DEFAULT_WIDTH = 100;
    private static final int DEFAULT_HEIGHT = 100;

    private static final String KEY_ENABLE_UML2 = "enableUML21";
    private static final String KEY_ENABLE_EA = "enableEA";
    private static final String KEY_ENABLE_MD = "enableMD";
    private static final boolean DEFAULT_ENABLE_UML21 = true;
    private static final boolean DEFAULT_ENABLE_EA = true;
    private static final boolean DEFAULT_ENABLE_MD = true;

    protected static final String KEY_HELP_OPEN = "helpOpen";
    protected static final boolean DEFAULT_HELP_OPEN = true;

    protected static final String KEY_MODE = "mode";
    public static final String KEY_FORMAT = "format";
    public static final String KEY_XMI_FILENAME = "xmi";
    protected static final String KEY_NEW_PROJECT_IMPORT = "newProjectImport"; // @See Mantis 0002492
    protected static final String KEY_WRITER = "writer";
    protected static final String KEY_READER = "reader";
    protected static final String KEY_LANGUAGE = "language";

    public static final String KEY_EXPORT_SEPARATE_FILE = "separateFile";
    public static final String KEY_AUTO_MERGE = "automaticMerge";
    public static final String KEY_UNIT_SELECTION = "units";
    public static final String KEY_SHOW_INFO_DIALOG = "showInfoDialog";// to silent export - import complete

    public static final String KEY_ADVANCED_LOGGER = "advancedLogger";// to silent export - import complete

    protected static final boolean DEFAULT_NEW_PROJECT_IMPORT = false;// @See Mantis 0002492
    protected static final boolean DEFAULT_SEPARATE_FILES = false;
    protected static final boolean DEFAULT_SHOW_INFO_DIALOG = true;// to silent export - import complete

    public static final String KEY_IS_RHAPSODYCL = "isRhapsodyCL";
    public static final String KEY_IS_NO_THREAD = "nothread";
    private Properties commandLineProperties;
    private Properties userProperties;
    private Properties installProperties;

    private MetamodelManager metamodelManager;
    private MDWorkbench mdw;
    private Metamodel selectedMetamodel;
    private ConnectorDescriptor selectedReader;
    private ConnectorDescriptor selectedWriter;
    private Model alreadyReadImportModel = null;
    private ImportToolKind customization = null;

    public enum importMethodKind {
        ADD("Write in current Project"), ADD_AND_REPLACE("Automatic merging"), DIFFMERGE("Manual merging");

        public final String literal;

        private importMethodKind(String literal) {
            this.literal = literal;
        }
    }

    private RhapsodyXMIToolkitParameters.importMethodKind importMethod = ImportMethodControl.getADD_METHOD();

    List<Object> selectedUnitsToExport = new ArrayList<Object>();
    List<Object> selectedNamespacesToImport;
    boolean allUnitsAreSelected = true;
    // PROPERTIES -------------------------------------------------------
    private RhapsodyPropertiesReader propertiesReader;
    // TAGS -------------------------------------------------------------
    private RhapsodyTagsReader tagsReader;
    private boolean exportInSeparateFiles = false;

    protected RhapsodyXMIToolkitParameters(MDWorkbench workbench) {
        this.mdw = workbench;
        this.setMetamodelFactory();
    }

    public RhapsodyXMIToolkitParameters() {
        this.setMetamodelFactory();
    }

    public MDWorkbench getMDWorkbench() {
        if (mdw == null) {
            try {
                mdw = MDWorkbenchFactory.create("rhapsody.xmi");
            } catch (CoreException e) {
                e.printStackTrace();
            }
        }
        return mdw;
    }

    private void setMetamodelFactory() {
        Thread.currentThread().setContextClassLoader(getClass().getClassLoader());
        metamodelManager = this.getMDWorkbench().getMetamodelManager();
    }

    public Properties getCommandLineProperties() {
        if (commandLineProperties == null) {
            commandLineProperties = new Properties();
        }
        return commandLineProperties;
    }

    protected Properties getInstallProperties() {
        if (installProperties == null) {
            installProperties = new Properties();
        }
        return installProperties;
    }

    public File getInstallPropertiesFile() {
        return new File(getResourcesDirectory(), "install.ini");
    }

    public File getLogDirectory() {
        File directory = new File(System.getProperty("user.home"), "IBM Rational Rhapsody XMI Toolkit");
        if (!directory.exists()) {
            directory.mkdirs();
        }
        return directory;
    }

    public Metamodel getMetamodelFromCommandLineFormat(UMLVersionKind metamodel) {
        for (Iterator i = getMetamodels().iterator(); i.hasNext();) {
            Metamodel meta = (Metamodel) i.next();
            if (meta.getUniqueID().equals(metamodel.metamodelID)) {
                return meta;
            }
        }

        return null;
    }

    public ConnectorDescriptor getWriterFromCommandLineFormat(String writerName) {

        try {
            return getSelectedMetamodel().getModelWriterDescriptor(writerName);
        } catch (CoreException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return null;
    }

    public Metamodel getMetamodelNamed(String metamodelName, String metamodelVersion) {
        String name = metamodelName.toUpperCase();
        String version = null;
        if (metamodelVersion != null) {
            version = metamodelVersion.toUpperCase();
        }

        for (Iterator i = getMetamodels().iterator(); i.hasNext();) {
            Metamodel meta = (Metamodel) i.next();
            if (meta.getName().equals(name)) {
                if ((meta.getVersion() == null) && (version == null)) {
                    return meta;
                } else if (!((meta.getVersion() == null) || (version == null))) {
                    if (meta.getVersion().equals(version)) {
                        return meta;
                    }
                }
            }
        }

        return null;
    }

    public Metamodel getMetamodelByID(String id) {
        try {
            return getMetamodelManager().getMetamodel(id);
        } catch (CoreException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return null;
    }

    public boolean isCommandLineProperty(String key) {
        return getCommandLineProperties().getProperty(key) != null;
    }

    private String getProperty(String key) {
        String value = getCommandLineProperties().getProperty(key);
        if (value != null) {
            return value;
        } else {
            return getUserProperties().getProperty(key);
        }
    }

    public String getProperty(String key, String defaultValue) {
        String value = getProperty(key);
        if (value != null) {
            return value;
        } else {
            return defaultValue;
        }
    }

    public boolean getProperty(String key, boolean defaultValue) {
        String value = getProperty(key);
        if (value != null) {
            return Boolean.valueOf(value).booleanValue();
        } else {
            return defaultValue;
        }
    }

    public int getProperty(String key, int defaultValue) {
        String value = getProperty(key);
        if (value != null) {
            return Integer.parseInt(value);
        } else {
            return defaultValue;
        }
    }

    public File getResourcesDirectory() {
        return new File(getToolkitDirectory(), "resources");
    }

    public ConnectorDescriptor getSelectedReader() {
        if (getSelectedMetamodel() != null) {
            if ((selectedReader == null) && !getSelectedMetamodel().getModelReaderDescriptors().isEmpty()) {
                if (getFormat() != null) {
                    try {
                        selectedReader = getSelectedMetamodel().getModelReaderDescriptor(getFormat().readerName);
                    } catch (CoreException e) {
                        // does not exist
                    }
                }
                if (selectedReader == null) {
                    selectedReader = getSelectedMetamodel().getModelReaderDescriptors().get(0);
                }
            }
        }
        return selectedReader;
    }

    public void setSelectedReader(ConnectorDescriptor reader) {
        if (!getSelectedMetamodel().getModelReaderDescriptors().contains(reader)) {
            throw new IllegalArgumentException(
                    "Illegal connector " + reader.getName() + "for " + getSelectedMetamodel().getPresentationName() + " metamodel.");
        } else {
            selectedReader = reader;
        }
    }

    public ConnectorDescriptor getSelectedWriter() {
        if (getSelectedMetamodel() != null) {
            if ((selectedWriter == null) && !getSelectedMetamodel().getModelWriterDescriptors().isEmpty()) {
                selectedWriter = getSelectedMetamodel().getModelWriterDescriptors().get(0);
            }
        }
        return selectedWriter;
    }

    public void setSelectedWriter(ConnectorDescriptor writer) {
        if (!getSelectedMetamodel().getModelWriterDescriptors().contains(writer)) {
            throw new IllegalArgumentException(
                    "Illegal connector " + writer.getName() + "for " + getSelectedMetamodel().getPresentationName() + " metamodel.");
        } else {
            selectedWriter = writer;
        }
    }

    public File getToolkitDirectory() {
        return new File(getToolkitPath());
    }

    protected String getToolkitPath() {
        return getProperty(KEY_TOOLKIT_DIRECTORY, System.getProperty("user.dir"));
    }

    public String getToolkitRelease() {
        String release = getInstallProperties().getProperty(KEY_RELEASE);
        if (release != null) {
            return release;
        } else {
            return "";
        }
    }

    public Properties getUserProperties() {
        if (userProperties == null) {
            userProperties = new Properties(getDefaultProperties()); // Adding management of default values
        }
        return userProperties;
    }

    protected Properties getDefaultProperties() {
        Properties defaultProperties = new Properties();
        defaultProperties.put(KEY_ENABLE_EA, DEFAULT_ENABLE_EA);
        defaultProperties.put(KEY_EXPORT_EMBEDDED_SYSML_PROFILE, DEFAULT_EXPORT_EMBEDDED_SYSML_PROFILE);
        return defaultProperties;
    }

    protected File getUserPropertiesFile() {
        return new File(getResourcesDirectory(), "toolkit.ini");
    }

    public boolean isDebugMode() {
        return getProperty(KEY_DEBUG_MODE, false);
    }

    public boolean isRhapsodyCL() {
        return getProperty(KEY_IS_RHAPSODYCL, false);
    }

    public boolean isNoThread() {
        return getProperty(KEY_IS_NO_THREAD, false);
    }

    public boolean isSilentMode() {
        return getProperty(KEY_SILENT_MODE, false);
    }

    public void save() throws IOException {
        OutputStream out = new FileOutputStream(getUserPropertiesFile());
        try {
            getUserProperties().store(out, "Toolkit parameters");

        } finally {
            out.close();
        }
    }

    public void setProperty(String key, String value) {
        setProperty_(key, value);
    }

    public void setProperty(String key, boolean value) {
        setProperty_(key, "" + value);
    }

    public void setProperty(String key, int value) {
        setProperty_(key, "" + value);
    }

    private void setProperty_(String key, String value) {
        String oldValue = getProperty(key);
        if ((oldValue == null) || (value == null) || !oldValue.equals(value)) {
            getUserProperties().setProperty(key, value);
            // getCommandLineProperties().setProperty(key, value);
        }
    }

    public MetamodelManager getMetamodelManager() {
        return metamodelManager;
    }

    public void read(String[] args) throws IOException, SAXException, IllegalArgumentException {
        // command line properties
        for (int i = 0; i < args.length; i += 2) {
            if ((i + 1) < args.length) {
                String key = args[i];
                if (key.startsWith("-")) {
                    key = key.substring(1);
                }
                String value = args[i + 1];

                getCommandLineProperties().setProperty(key, value);
            }
        }
        // check automerge
        if ("true".equalsIgnoreCase(getCommandLineProperties().getProperty(KEY_AUTO_MERGE))) {
            importMethod = importMethodKind.ADD_AND_REPLACE;
        }

        // install properties
        try {
            getInstallProperties().load(new FileInputStream(getInstallPropertiesFile()));
        } catch (IOException e) {
        }

        // user properties
        try {
            getUserProperties().load(new FileInputStream(getUserPropertiesFile()));
        } catch (IOException e) {
        }

        this.readSilentParameters();

        try {
            propertiesReader = new RhapsodyPropertiesReader(getToolkitDirectory() + File.separator + DEFAULT_RHAPSODY_PROPERTIES_FILE);
        } catch (IOException e) {
            propertiesReader = RhapsodyPropertiesReader.EMPTY_PROPERTIES_READER;
            System.out.println("Error reading property file.");
            e.printStackTrace();
        }

        try {
            tagsReader = new RhapsodyTagsReader(getToolkitDirectory() + File.separator + DEFAULT_RHAPSODY_TAGS_FILE);
        } catch (IOException e) {
            tagsReader = RhapsodyTagsReader.EMPTY_TAGS_READER;
            System.out.println("Error reading tags file.");
            e.printStackTrace();
        }
    }

    protected void readSilentParameters() {
        // silent file ?
        String silentFile = getSilentFilename();
        if (silentFile != null) {
            if (!new File(silentFile).exists() || new File(silentFile).isDirectory()) {
                throw new IllegalArgumentException("Silent file does not exist or is a directory");
            }
            Properties silentProperties = new Properties();
            try {
                silentProperties.load(new FileInputStream(silentFile));
            } catch (IOException e) {
                System.out.println("Error reading silent parameter file.");
                e.printStackTrace();
            }
            for (Object element : silentProperties.entrySet()) {
                Map.Entry entry = (Map.Entry) element;
                getCommandLineProperties().put(entry.getKey(), entry.getValue());
            }
        }
    }

    public void checkSilentParameters() {
        try {
            // mode
            String mode = getCommandLineProperties().getProperty(KEY_MODE);
            if ((mode == null) || (mode.length() == 0)) {
                throw new IllegalArgumentException("Missing " + KEY_MODE + " parameter");
            }
            if (!mode.equalsIgnoreCase(XMIMode.XMI_IMPORT.toString()) && !mode.equalsIgnoreCase(XMIMode.XMI_EXPORT.toString())
                    && !mode.equalsIgnoreCase(XMIMode.TAU_IMPORT.toString()) && !mode.equalsIgnoreCase(XMIMode.TAU_EXPORT.toString())) {
                throw new IllegalArgumentException("Unknown \"" + mode + "\" " + KEY_MODE + " parameter");
            }

            // format
            UMLVersionKind format = getFormat(getCommandLineProperties().getProperty(KEY_FORMAT));
            if (format == null) {
                throw new IllegalArgumentException("Missing or unknown " + KEY_FORMAT + " parameter");
            }
            Metamodel metamodel = getMetamodelFromCommandLineFormat(format);
            if (metamodel == null) {
                throw new IllegalArgumentException("Unknown \"" + format.commandLineID + "\" " + KEY_FORMAT + " parameter");
            } else {
                setSelectedMetamodel(metamodel, format);
            }

            // EXPORT Kind
            String isSeparateFileString = getCommandLineProperties().getProperty(KEY_EXPORT_SEPARATE_FILE, "false");
            if ("true".equalsIgnoreCase(isSeparateFileString)) {
                exportInSeparateFiles = true;
            }

            // XMI file
            String xmiFile = getCommandLineProperties().getProperty(KEY_XMI_FILENAME);
            if ((xmiFile == null) || (xmiFile.length() == 0)) {
                throw new IllegalArgumentException("Missing " + KEY_XMI_FILENAME + " parameter");
            }
            if (!exportInSeparateFiles && new File(xmiFile).exists() && new File(xmiFile).isDirectory()) {
                throw new IllegalArgumentException("The specified xmi file is a directory");
            }
            if (exportInSeparateFiles && new File(xmiFile).exists() && new File(xmiFile).isFile()) {
                throw new IllegalArgumentException("The specified xmi file is a directory");
            }

            // IMPORT Method
            String importMethodString = getCommandLineProperties().getProperty(KEY_AUTO_MERGE, "false");
            if ("true".equalsIgnoreCase(importMethodString)) {
                importMethod = importMethodKind.ADD_AND_REPLACE;
            }

            // Unit selection
            // String unitSelectionString=getCommandLineProperties().getProperty(KEY_UNIT_SELECTION, "");
            // if(unitSelectionString.length()>0)
            // {
            // if("uml21".equals(getSelectedMetamodel().getUniqueID()))
            // {
            // if("export".equalsIgnoreCase(mode))
            // {
            //
            // }
            // }
            // }
            // Properties file
            String rhapsodyPropertiesFile = getCommandLineProperties().getProperty(KEY_RHAPSODY_PROPERTIES_FILE);
            // if properties file parameter present
            if (rhapsodyPropertiesFile != null) {
                if (rhapsodyPropertiesFile.length() != 0) {
                    // check that it is not a directory
                    if (new File(rhapsodyPropertiesFile).exists() && new File(rhapsodyPropertiesFile).isDirectory()) {
                        throw new IllegalArgumentException("The specified properties file is a directory");
                    } else {
                        try {
                            propertiesReader = new RhapsodyPropertiesReader(rhapsodyPropertiesFile);
                        } catch (IOException e) {
                            propertiesReader = RhapsodyPropertiesReader.EMPTY_PROPERTIES_READER;
                            System.out.println("Error reading property file.");
                            e.printStackTrace();
                        }
                    }
                }
            } else {

                try {
                    propertiesReader = new RhapsodyPropertiesReader(getToolkitDirectory() + File.separator + DEFAULT_RHAPSODY_PROPERTIES_FILE);
                } catch (IOException e) {
                    propertiesReader = RhapsodyPropertiesReader.EMPTY_PROPERTIES_READER;
                    System.out.println("Error reading property file.");
                    e.printStackTrace();
                }
            }

            // Tags file
            String rhapsodyTagsFile = getCommandLineProperties().getProperty(KEY_RHAPSODY_TAGS_FILE);
            // if tags file parameter present
            if (rhapsodyTagsFile != null) {
                if (rhapsodyTagsFile.length() != 0) {
                    // check that it is not a directory
                    if (new File(rhapsodyTagsFile).exists() && new File(rhapsodyTagsFile).isDirectory()) {
                        throw new IllegalArgumentException("The specified tags file is a directory");
                    } else {
                        try {
                            tagsReader = new RhapsodyTagsReader(rhapsodyTagsFile);
                        } catch (IOException e) {
                            tagsReader = RhapsodyTagsReader.EMPTY_TAGS_READER;
                            System.out.println("Error reading tags file.");
                            e.printStackTrace();
                        }
                    }
                }
            } else {
                try {
                    tagsReader = new RhapsodyTagsReader(getToolkitDirectory() + File.separator + DEFAULT_RHAPSODY_TAGS_FILE);
                } catch (IOException e) {
                    tagsReader = RhapsodyTagsReader.EMPTY_TAGS_READER;
                    System.out.println("Error reading tags file.");
                    e.printStackTrace();
                }
            }

        } catch (IllegalArgumentException e) {
            StringBuffer metamodels = new StringBuffer();
            for (Iterator i = getMetamodels().iterator(); i.hasNext();) {
                metamodels.append(((Metamodel) i.next()).getUniqueID());
                if (i.hasNext()) {
                    metamodels.append("|");
                }
            }

            StringBuffer buffer = new StringBuffer(e.getMessage());
            buffer.append("\n\nExpected parameters : \n");
            buffer.append("-" + KEY_MODE + " <mode> ");
            buffer.append("-" + KEY_FORMAT + " <format> ");
            buffer.append("-" + KEY_XMI_FILENAME + " <file> ");
            buffer.append("[-" + KEY_RHAPSODY_PROPERTIES_FILE + " <file>]");
            buffer.append("[-" + KEY_RHAPSODY_TAGS_FILE + " <file>]");
            buffer.append("\nOr :\n");
            buffer.append("-" + KEY_SILENT_FILENAME + " <file>");
            buffer.append("\n\n");

            buffer.append("Parameter details :\n");
            buffer.append("-" + KEY_MODE + "       : Determines wheter the toolkit runs into import or export mode (" + XMIMode.XMI_IMPORT + "|"
                    + XMIMode.XMI_EXPORT + "|" + XMIMode.TAU_IMPORT + "|" + XMIMode.TAU_EXPORT + ")\n");
            buffer.append("-" + KEY_FORMAT + "     : The metamodel to be used (" + metamodels + ")\n");
            buffer.append("-" + KEY_WRITER + "     : The connector to use for writing\n");
            buffer.append("-" + KEY_READER + "     : The connector to use for reading\n");
            buffer.append("-" + KEY_XMI_FILENAME + "        : The filename where to load or write data (\"c:\\myModel.xmi\")\n");
            buffer.append("-" + KEY_RHAPSODY_PROPERTIES_FILE + " Indicates the file listing the properties to export\n");
            buffer.append("-" + KEY_RHAPSODY_TAGS_FILE + " : Indicates the file listing the tags defined in profiles to export\n");

            buffer.append("-" + KEY_SILENT_FILENAME + " : a Java property file which contains all the above parameters\n");

            throw new IllegalArgumentException(buffer.toString());
        }
    }

    public int getPartialExportCollectionDispathID() {
        return getProperty(KEY_EXPORT_PARTIAL_ID, DEFAULT_EXPORT_PARTIAL_ID);
    }

    public boolean getExportPartial() {
        return getPartialExportCollectionDispathID() != 0;
    }

    public boolean getExportAnnotations() {
        return getProperty(KEY_EXPORT_ANNOTATIONS, DEFAULT_EXPORT_ANNOTATIONS);
    }

    public boolean getExportDiagramsAnnotations() {
        return getProperty(KEY_EXPORT_DIAGRAMS_ANNOTATIONS, DEFAULT_EXPORT_DIAGRAMS_ANNOTATIONS);
    }

    public boolean getExportComments() {
        return getProperty(KEY_EXPORT_COMMENTS, DEFAULT_EXPORT_COMMENTS);
    }

    public boolean getExportRequirements() {
        return getProperty(KEY_EXPORT_REQUIREMENTS, DEFAULT_EXPORT_REQUIREMENTS);
    }

    public boolean getExportConstraints() {
        return getProperty(KEY_EXPORT_CONSTRAINTS, DEFAULT_EXPORT_CONSTRAINTS);
    }

    public boolean getExportCollaborations() {
        return getProperty(KEY_EXPORT_COLLABORATIONS, DEFAULT_EXPORT_COLLABORATIONS);
    }

    public boolean getExportPackageLinks() {
        return getProperty(KEY_EXPORT_PACKAGE_LINKS, DEFAULT_EXPORT_PACKAGE_LINKS);
    }

    public boolean getExportDeployments() {
        return getProperty(KEY_EXPORT_DEPLOYMENTS, DEFAULT_EXPORT_DEPLOYMENTS);
    }

    public boolean getExportDescriptionHTML() {
        return getProperty(KEY_EXPORT_DESCRIPTION_HTML, DEFAULT_EXPORT_DESCRIPTION_HTML);
    }

    public boolean getExportDescriptionRTF() {
        return getProperty(KEY_EXPORT_DESCRIPTION_RTF, DEFAULT_EXPORT_DESCRIPTION_RTF);
    }

    public boolean getExportMethods() {
        return getProperty(KEY_EXPORT_METHODS, DEFAULT_EXPORT_METHODS);
    }

    public boolean getExportProperties() {
        return getProperty(KEY_EXPORT_PROPERTIES, DEFAULT_EXPORT_PROPERTIES);
    }

    public boolean getExportTags() {
        return getProperty(KEY_EXPORT_TAGS, DEFAULT_EXPORT_TAGS);
    }

    public boolean getImportStatechart() {
        return getProperty(KEY_IMPORT_STATECHART, DEFAULT_IMPORT_STATECHART);
    }

    public boolean getImportSequenceDiagram() {
        return getProperty(KEY_IMPORT_SEQUENCE_DIAGRAM, DEFAULT_IMPORT_SEQUENCE_DIAGRAM);
    }

    public boolean getImportClassDiagram() {
        return getProperty(KEY_IMPORT_CLASS_DIAGRAM, DEFAULT_IMPORT_CLASS_DIAGRAM);
    }

    public boolean getImportUseCaseDiagram() {
        return getProperty(KEY_IMPORT_USECASE_DIAGRAM, DEFAULT_IMPORT_USECASE_DIAGRAM);
    }

    public boolean getImportComponentDiagram() {
        return getProperty(KEY_IMPORT_COMPONENT_DIAGRAM, DEFAULT_IMPORT_COMPONENT_DIAGRAM);
    }

    public boolean getImportDeploymentDiagram() {
        return getProperty(KEY_IMPORT_DEPLOYMENT_DIAGRAM, DEFAULT_IMPORT_DEPLOYMENT_DIAGRAM);
    }

    public boolean getImportCollaborationDiagram() {
        return getProperty(KEY_IMPORT_COLLABORATION_DIAGRAM, DEFAULT_IMPORT_COLLABORATION_DIAGRAM);
    }

    public boolean getImportStructureDiagram() {
        return getProperty(KEY_IMPORT_STRUCTURE_DIAGRAM, DEFAULT_IMPORT_STRUCTURE_DIAGRAM);
    }

    public boolean getImportActivityDiagram() {
        return getProperty(KEY_IMPORT_ACTIVITY_DIAGRAM, DEFAULT_IMPORT_ACTIVITY_DIAGRAM);
    }

    public boolean getExportRhapsodyExtension() {
        return getProperty(KEY_EXPORT_WITH_RHAPSODY_EXTENSION, DEFAULT_EXPORT_WITH_RHAPSODY_EXTENSION);
    }

    public boolean getExportStatemachines() {
        return getProperty(KEY_EXPORT_STATEMACHINES, DEFAULT_EXPORT_STATEMACHINES);
    }

    public boolean getImportCollaborations() {
        return getProperty(KEY_IMPORT_COLLABORATIONS, DEFAULT_IMPORT_COLLABORATIONS);
    }

    public boolean getImportCompositionsAsParts() {
        return getProperty(KEY_IMPORT_COMPOSITIONS_AS_PARTS, DEFAULT_IMPORT_COMPOSITION_AS_PARTS);
    }

    public boolean getImportDeployments() {
        return getProperty(KEY_IMPORT_DEPLOYMENTS, DEFAULT_IMPORT_DEPLOYMENTS);
    }

    public boolean getImportStatemachines() {
        return getProperty(KEY_IMPORT_STATEMACHINES, DEFAULT_IMPORT_STATEMACHINES);
    }

    public String getAssociationStyle() {
        return getProperty(KEY_ASSOCIATION_STYLE, DEFAULT_ASSOCIATION_STYLE);
    }

    public String getNoteStyle() {
        return getProperty(KEY_NOTE_STYLE, DEFAULT_NOTE_STYLE);
    }

    public String getHorizontalSynchStyle() {
        return getProperty(KEY_HORIZONTAL_SYNCH_STYLE, DEFAULT_HORIZONTAL_SYNCH_STYLE);
    }

    public String getVerticalSynchStyle() {
        return getProperty(KEY_VERTICAL_SYNCH_STYLE, DEFAULT_VERTICAL_SYNCH_STYLE);
    }

    public String getToolName() {
        return getProperty(KEY_TOOL_NAME, DEFAULT_TOOL_NAME);
    }

    public int getXScalingFactor() {
        return getProperty(KEY_X_SCALING_FACTOR, DEFAULT_X_SCALING_FACTOR);
    }

    public int getYScalingFactor() {
        return getProperty(KEY_Y_SCALING_FACTOR, DEFAULT_Y_SCALING_FACTOR);
    }

    public int getWidthScalingFactor() {
        return getProperty(KEY_WIDTH_SCALING_FACTOR, DEFAULT_WIDTH_SCALING_FACTOR);
    }

    public int getHeightScalingFactor() {
        return getProperty(KEY_HEIGHT_SCALING_FACTOR, DEFAULT_HEIGHT_SCALING_FACTOR);
    }

    public int getDefaultHeight() {
        return getProperty(KEY_DEFAULT_HEIGHT, DEFAULT_HEIGHT);
    }

    public int getDefaultWidth() {
        return getProperty(KEY_DEFAULT_WIDTH, DEFAULT_WIDTH);
    }

    public boolean getImportSubstatesAsNestedChart() {
        return getProperty(KEY_IMPORT_SUBSTATES_AS_NESTEDCHART, DEFAULT_IMPORT_SUBSTATES_AS_NESTEDCHART);
    }

    public String getPropertiesFile() {
        return getProperty(KEY_RHAPSODY_PROPERTIES_FILE, DEFAULT_RHAPSODY_PROPERTIES_FILE);
    }

    public String getTagsFile() {
        return getProperty(KEY_RHAPSODY_TAGS_FILE, DEFAULT_RHAPSODY_TAGS_FILE);
    }

    public RhapsodyPropertiesReader getRhapsodyPropertiesReader() {
        return propertiesReader;
    }

    public RhapsodyTagsReader getRhapsodyTagsReader() {
        return tagsReader;
    }

    public String getSilentFilename() {
        return getCommandLineProperties().getProperty(KEY_SILENT_FILENAME);
    }

    public boolean isRhapsodyProperty(String propertyName) {
        int i = propertyName.indexOf('.');
        if (i < 0) {
            return false;
        } else {
            i = propertyName.indexOf('.', i + 1);
            if (i < 0) {
                return false;
            } else {
                return propertyName.indexOf('.', i + 1) < 0;
            }
        }
    }

    public void setPartialExportCollectionDispathID(int value) {
        setProperty(KEY_EXPORT_PARTIAL_ID, value);
    }

    public void setExportAnnotations(boolean value) {
        setProperty(KEY_EXPORT_ANNOTATIONS, value);
    }

    public void setExportDiagramsAnnotations(boolean value) {
        setProperty(KEY_EXPORT_DIAGRAMS_ANNOTATIONS, value);
    }

    public void setExportComments(boolean value) {
        setProperty(KEY_EXPORT_COMMENTS, value);
    }

    public void setExportRequirements(boolean value) {
        setProperty(KEY_EXPORT_REQUIREMENTS, value);
    }

    public void setExportConstraints(boolean value) {
        setProperty(KEY_EXPORT_CONSTRAINTS, value);
    }

    public void setExportCollaborations(boolean value) {
        setProperty(KEY_EXPORT_COLLABORATIONS, value);
    }

    public void setExportPackageLinks(boolean value) {
        setProperty(KEY_EXPORT_PACKAGE_LINKS, value);
    }

    public void setExportDeployments(boolean value) {
        setProperty(KEY_EXPORT_DEPLOYMENTS, value);
    }

    public void setExportMethods(boolean value) {
        setProperty(KEY_EXPORT_METHODS, value);
    }

    public void setExportProperties(boolean value) {
        setProperty(KEY_EXPORT_PROPERTIES, value);
    }

    public void setExportTags(boolean value) {
        setProperty(KEY_EXPORT_TAGS, value);
    }

    public void setExportStatemachines(boolean value) {
        setProperty(KEY_EXPORT_STATEMACHINES, value);
    }

    public void setExportDescriptionHTML(boolean value) {
        setProperty(KEY_EXPORT_DESCRIPTION_HTML, value);
    }

    public void setExportDescriptionRTF(boolean value) {
        setProperty(KEY_EXPORT_DESCRIPTION_RTF, value);
    }

    public void setImportCollaborations(boolean value) {
        setProperty(KEY_IMPORT_COLLABORATIONS, value);
    }

    public void setImportDeployments(boolean value) {
        setProperty(KEY_IMPORT_DEPLOYMENTS, value);
    }

    public void setImportStatemachines(boolean value) {
        setProperty(KEY_IMPORT_STATEMACHINES, value);
    }

    public void setImportSubstatesAsNestedChart(boolean value) {
        setProperty(KEY_IMPORT_SUBSTATES_AS_NESTEDCHART, value);
    }

    public void setImportStatechart(boolean value) {
        setProperty(KEY_IMPORT_STATECHART, value);
    }

    public void setImportSequenceDiagram(boolean value) {
        setProperty(KEY_IMPORT_SEQUENCE_DIAGRAM, value);
    }

    public void setImportClassDiagram(boolean value) {
        setProperty(KEY_IMPORT_CLASS_DIAGRAM, value);
    }

    public void setImportUseCaseDiagram(boolean value) {
        setProperty(KEY_IMPORT_USECASE_DIAGRAM, value);
    }

    public void setImportComponentDiagram(boolean value) {
        setProperty(KEY_IMPORT_COMPONENT_DIAGRAM, value);
    }

    public void setImportDeploymentDiagram(boolean value) {
        setProperty(KEY_IMPORT_DEPLOYMENT_DIAGRAM, value);
    }

    public void setImportCollaborationDiagram(boolean value) {
        setProperty(KEY_IMPORT_COLLABORATION_DIAGRAM, value);
    }

    public void setImportStructureDiagram(boolean value) {
        setProperty(KEY_IMPORT_STRUCTURE_DIAGRAM, value);
    }

    public void setImportActivityDiagram(boolean value) {
        setProperty(KEY_IMPORT_ACTIVITY_DIAGRAM, value);
    }

    public boolean getCreateClassForSignal() {
        return getProperty(KEY_CREATE_CLASS_FOR_SIGNAL, DEFAULT_CREATE_CLASS_FOR_SIGNAL);
    }

    public void setCreateClassForSignal(boolean value) {
        setProperty(KEY_CREATE_CLASS_FOR_SIGNAL, value);
    }

    public boolean getIsPlugin() {
        return getPluginRhapsodyID() != 0;
    }

    public int getPluginRhapsodyID() {
        return getProperty(KEY_AS_PLUGIN, 0);
    }

    public boolean getExportEmbeddedSysMLProfile() {
        return getProperty(KEY_EXPORT_EMBEDDED_SYSML_PROFILE, DEFAULT_EXPORT_EMBEDDED_SYSML_PROFILE);
    }

    public void setExportEmbeddedSysMLProfile(boolean exportEmbeddedSysMLProfile) {
        setProperty(KEY_EXPORT_EMBEDDED_SYSML_PROFILE, exportEmbeddedSysMLProfile);
        getDefaultProperties().put(KEY_EXPORT_EMBEDDED_SYSML_PROFILE, exportEmbeddedSysMLProfile); // In order to never
    }

    // getExportRhapsodyExtension
    public void setExportRhapsodyExtension(boolean exportRhapsodyProfiles) {
        setProperty(KEY_EXPORT_WITH_RHAPSODY_EXTENSION, exportRhapsodyProfiles);
    }

    public boolean getAddRoleNames() {
        return getProperty(KEY_ADD_ROLE_NAMES, DEFAULT_ADD_ROLE_NAMES);
    }

    public void setAddRoleNames(boolean value) {
        setProperty(KEY_ADD_ROLE_NAMES, value);
    }

    public boolean getUseTauCompatibleMessages() {
        return getProperty(KEY_USE_TAU_COMPATIBLE_MESSAGES, DEFAULT_USE_TAU_COMPATIBLE_MESSAGES);
    }

    public void setUseTauCompatibleMessages(boolean value) {
        setProperty(KEY_USE_TAU_COMPATIBLE_MESSAGES, value);
    }

    public boolean getRealizeAllMessages() {
        return getProperty(KEY_REALIZE_ALL_MESSAGES, DEFAULT_REALIZE_ALL_MESSAGES);
    }

    public void setRealizeAllMessages(boolean value) {
        setProperty(KEY_REALIZE_ALL_MESSAGES, value);
    }

    public boolean getDontCreateObjects() {
        return getProperty(KEY_DONT_CREATE_OBJECTS, DEFAULT_DONT_CREATE_OBJECTS);
    }

    public void setDontCreateObjects(boolean value) {
        setProperty(KEY_DONT_CREATE_OBJECTS, value);
    }

    public boolean getAddNewLineForAction() {
        return getProperty(KEY_ADD_NEW_LINE_FOR_ACTION, DEFAULT_ADD_NEW_LINE_FOR_ACTION);
    }

    public void setAddNewLineForAction(boolean value) {
        setProperty(KEY_ADD_NEW_LINE_FOR_ACTION, value);
    }

    public boolean getEnableUML21() {
        return getProperty(KEY_ENABLE_UML2, DEFAULT_ENABLE_UML21);
    }

    public void setEnableUML21(boolean value) {
        setProperty(KEY_ENABLE_UML2, value);
    }

    public boolean getEnableUMLEA() {
        return getProperty(KEY_ENABLE_EA, DEFAULT_ENABLE_EA);
    }

    public void setEnableUMLEA(boolean value) {
        setProperty(KEY_ENABLE_EA, value);
    }

    public boolean getEnableUMLMD() {
        return getProperty(KEY_ENABLE_MD, DEFAULT_ENABLE_MD);
    }

    public void setEnableUMLMD(boolean value) {
        setProperty(KEY_ENABLE_MD, value);
    }

    public boolean getCreateClassifiersForUnrealizedClassifierRoles() {
        return getProperty(KEY_CREATE_CLASSIFIERS_FOR_UNREALIZED_CLASSIFIER_ROLE, DEFAULT_CREATE_CLASSIFIERS_FOR_UNREALIZED_CLASSIFIER_ROLE);
    }

    public void setCreateClassifiersForUnrealizedClassifierRoles(boolean value) {
        setProperty(KEY_CREATE_CLASSIFIERS_FOR_UNREALIZED_CLASSIFIER_ROLE, value);
    }

    public Collection getMetamodels() {
        Collection mms = new ArrayList(getMetamodelManager().getMetamodels());
        mms.remove(this.getMetamodelByID("rhapsody"));
        return mms;
    }

    public List getSelectedMetamodelWriterDescriptors() {
        Metamodel metamodel = getSelectedMetamodel();
        List modelWriterDescriptorsList = metamodel.getModelWriterDescriptors();
        if (metamodel.getUniqueID().equals("uml21")) {
            List listTmp = new ArrayList();
            Iterator writerIterator = modelWriterDescriptorsList.iterator();
            while (writerIterator.hasNext()) {
                ConnectorDescriptor writerDescriptor = (ConnectorDescriptor) writerIterator.next();
                if (!(writerDescriptor.getName().equals("XMI 2.1 (Diagram Interchange)"))) {
                    listTmp.add(writerDescriptor);
                }
            }
            return listTmp;
        } else {
            return modelWriterDescriptorsList;
        }
    }

    // @See Mantis 0002492
    public boolean canNewProjectImport() {
        return getSelectedMetamodel().getUniqueID().equals("uml21") && (isImportMode());
    }

    public boolean canUseLanguage() {
        return isImportMode();
    }

    public boolean canSelectConnector() {
        return !(isTauImportMode() || isTauExportMode());
    }

    public boolean canSelectFormat() {
        return !(isTauImportMode() || isTauExportMode());
    }

    public boolean isWithAdditionalDiagramFile(String filename) {
        File f = getAdditionalDiagramFile(filename);
        if ((f != null) && f.exists()) {
            return true;
        }
        return false;
    }

    public boolean isWithAdditionalDiagramFile() {
        return isWithAdditionalDiagramFile(getXMIFilename());
    }

    public File getAdditionalDiagramFile(String filename) {
        try {
            File file = new File(filename);
            File parent = file.getParentFile();
            return new File(parent, file.getName() + "." + EXTENSION_DIAGRAM_FILE);
        } catch (Exception e) {
        }
        return null;
    }

    // @See Mantis 0002492
    public boolean getNewProjectImport() {
        return (getProperty(KEY_NEW_PROJECT_IMPORT, DEFAULT_NEW_PROJECT_IMPORT));
    }

    public UMLVersionKind getFormat() {
        String umlLiteral = getProperty(KEY_FORMAT, "");
        return getFormat(umlLiteral);
    }

    public ImportToolKind getCustomization() {
        if (customization == null) {
            String umlLiteral = getProperty(KEY_FORMAT, "");
            getCommandLineProperties().remove(KEY_FORMAT);
            customization = getCustomization(umlLiteral);
        }
        return customization;
    }

    private ImportToolKind getCustomization(String umlLiteral) {
        for (ImportToolKind version : ImportToolKind.values()) {
            if (version.format.equalsIgnoreCase(umlLiteral)) {
                return version;
            }
        }
        return ImportToolKind.XMI;
    }

    public UMLVersionKind getFormat(String umlLiteral) {
        for (UMLVersionKind version : UMLVersionKind.values()) {
            if (version.commandLineID.equalsIgnoreCase(umlLiteral)) {
                return version;
            }
        }
        return null;
    }

    public String getWriter() {
        return getProperty(KEY_WRITER, "");
    }

    public String getReader() {
        return getProperty(KEY_READER, "");
    }

    public String getLanguage() {
        if (canUseLanguage()) {
            return getProperty("KEY_LANGUAGE", "");
        } else {
            return "";
        }
    }

    public XMIMode getMode() {
        XMIMode mode = XMIMode.getMode(getProperty(KEY_MODE, ""));
        if (mode != null) {
            return mode;
        } else {
            return XMIMode.XMI_IMPORT;
        }
    }

    public Model getAlreadyReadImportModel() {
        return alreadyReadImportModel;
    }

    public void setAlreadyReadImportModel(Model model) {
        this.alreadyReadImportModel = model;
    }

    public Metamodel getSelectedMetamodel() {
        if (isTauImportMode() || isTauExportMode()) {
            selectedMetamodel = getMetamodelByID("uml21");
        } else {
            if ((selectedMetamodel == null) && !getMetamodels().isEmpty()) {
                selectedMetamodel = getMetamodelByID("uml21");
            }
        }
        return selectedMetamodel;
    }

    public String getXMIFilename() {
        return getProperty(KEY_XMI_FILENAME, "");
    }

    public boolean isXMIExportMode() {
        return getMode() == XMIMode.XMI_EXPORT;
    }

    public boolean isXMIImportMode() {
        return getMode() == XMIMode.XMI_IMPORT;
    }

    public boolean isImportMode() {
        return (getMode() == XMIMode.XMI_IMPORT) || (getMode() == XMIMode.TAU_IMPORT);
    }

    public boolean isExportMode() {
        return (getMode() == XMIMode.XMI_EXPORT) || (getMode() == XMIMode.TAU_EXPORT);
    }

    public boolean isTauExportMode() {
        return getMode() == XMIMode.TAU_EXPORT;
    }

    public boolean isTauImportMode() {
        return getMode() == XMIMode.TAU_IMPORT;
    }

    protected void loadMetamodels() throws IOException, SAXException {
        // select one metamodel
        UMLVersionKind format = getFormat();
        this.setSelectedMetamodel(getMetamodelFromCommandLineFormat(format), format);

        // set writer
        String writerStr = getWriter();
        if (!writerStr.equals("")) {
            ConnectorDescriptor desc = getWriterFromCommandLineFormat(writerStr);
            if (desc != null) {
                this.setSelectedWriter(desc);
            }
        }
    }

    // @See Mantis 0002492
    public void setNewProjectImport(boolean newProjectActivated) {
        setProperty(KEY_NEW_PROJECT_IMPORT, newProjectActivated);
    }

    public void setXMIExportMode() {
        setMode(XMIMode.XMI_EXPORT);
    }

    public void setImportMode() {
        setMode(XMIMode.XMI_IMPORT);
    }

    public void setMode(XMIMode mode) {
        setProperty(KEY_MODE, mode.toString());
    }

    public void setReader(String reader) {
        setProperty(KEY_READER, reader);
    }

    public void setWriter(String writer) {
        setProperty(KEY_WRITER, writer);
    }

    public void setLanguage(String language) {
        setProperty("KEY_LANGUAGE", language);
    }

    public void setSelectedMetamodel(Metamodel metamodel) {
        this.selectedMetamodel = metamodel;
        if (metamodel != null) {
            setProperty(KEY_FORMAT, metamodel.getUniqueID());
        }
    }

    public void setSelectedMetamodel(Metamodel metamodel, UMLVersionKind umlVersion) {
        this.selectedMetamodel = metamodel;
        if (metamodel != null) {
            setProperty(KEY_FORMAT, umlVersion.commandLineID);
        }
    }

    public void setXMIFilename(String filename) {
        if (filename == null) {
            filename = "";
        }
        setProperty(KEY_XMI_FILENAME, filename);
    }

    public void setExportInSeparateFiles(boolean exportInSeparateFiles) {
        this.exportInSeparateFiles = exportInSeparateFiles;
    }

    public boolean getExportInSeparateFiles() {
        return getProperty(KEY_EXPORT_SEPARATE_FILE, exportInSeparateFiles);
    }

    public List<Object> getSelectedUnitsToExport() {
        return selectedUnitsToExport;
    }

    public boolean getAllUnitsAreSelected() {
        return allUnitsAreSelected;
    }

    public void setAllUnitsAreSelected(boolean allUnitsAreSelected) {
        this.allUnitsAreSelected = allUnitsAreSelected;
    }

    public void setSelectedUnitsToExport(List<Object> selectedUnitsToExport) {
        this.selectedUnitsToExport = selectedUnitsToExport;
    }

    public List<Object> getSelectedNamespacesToImport() {
        return selectedNamespacesToImport;
    }

    public void setSelectedNamespacesToImport(List<Object> selectedNamespacesToImport) {
        this.selectedNamespacesToImport = selectedNamespacesToImport;
    }

    public RhapsodyXMIToolkitParameters.importMethodKind getImportMethod() {
        return this.importMethod;
    }

    public void setImportMethod(RhapsodyXMIToolkitParameters.importMethodKind importMethod) {
        this.importMethod = importMethod;
    }

    public void setHelpIsOpen(boolean isOpen) {
        setProperty(KEY_HELP_OPEN, isOpen);
    }

    public boolean getHelpIsOpen() {
        return getProperty(KEY_HELP_OPEN, DEFAULT_HELP_OPEN);
    }

    public boolean getAdvancedLogger() {
        return getProperty(KEY_ADVANCED_LOGGER, false);
    }

}