package com.sodius.rhapsody.toolkit.ui;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

public abstract class AdvancedBlocManager extends Composite {

	protected Button advanced;
	protected String titleLabel;
	protected Group advancedGroup;
	protected Label title;
	protected ToolkitUIManager manager;
	protected Composite rootComposite;
	protected ToolkitWizardSelectionPage page;
	protected SelectionListener listener;

	protected Map<String, Button> properties = new HashMap<String, Button>();

	private int advancedGroupHeight = -1;
	private int advancedGroupWidth = -1;

	public AdvancedBlocManager(Composite parent, ToolkitWizardSelectionPage page, ToolkitUIManager manager, String titleLabel) {
		super(parent, SWT.NONE);
		this.page = page;
		this.manager = manager;
		this.titleLabel = titleLabel;
	}

	protected void createContent() {

		createFields();
		createAndResetListener();
		registerListener();

	}

	protected abstract void createAdvancedProperties();

	protected abstract void registerListener();

	protected abstract void createFields();

	protected abstract void createAndResetListener();

	protected void createAdvancedButton() {
		advanced = new Button(this, SWT.BUTTON5);
		advanced.setText("Advanced >>");
		advanced.setLayoutData(new GridData());
		this.layout(true, true);
		advanced.addSelectionListener(listener);
		advanced.addHelpListener(manager.getHelpListener());
	}

	protected void clearAdvancedProperties() {
		advanced.setText("Advanced >>");
		((GridData) advancedGroup.getLayoutData()).exclude = true;
		advancedGroup.setVisible(false);
		this.layout();
	}

	protected void showAdvancedButton() {
		advanced.setText("Advanced >>");
		((GridData) advanced.getLayoutData()).exclude = false;
		advanced.setVisible(true);
		this.layout();
	}
	
	protected void showAdvancedProperties() {
		advanced.setText("<< Advanced");
		((GridData) advancedGroup.getLayoutData()).exclude = false;
		advancedGroup.setVisible(true);
		this.layout();
	}

	protected Button createCheckButton(Composite parent, String text, String parameterKey, boolean defaultParameterKey) {
		Button button = new Button(parent, SWT.CHECK);
		button.setText(text);
		properties.put(parameterKey, button);
		button.setSelection(manager.getToolkit().getParameters().getProperty(parameterKey, defaultParameterKey));
		return button;
	}

	protected void clearAdvancedButton() {
		advanced.setVisible(false);
		((GridData) advanced.getLayoutData()).exclude = true;
		this.layout(true, true);
	}

	protected boolean getAdvancedPropertyValue(String property) {
		if (properties.get(property) != null) {
			Button pButton = properties.get(property);
			return pButton.getSelection();
		} else
			return false;
	}

	protected void handleAdvancedButtonSelect() {
		if (advancedGroup==null)	return;
		Shell shell = page.getShell();
		Point shellSize = shell.getSize();
		Composite composite = (Composite) page.getControl();

		if (!advancedGroup.isVisible()) {
			if (advancedGroupHeight != 0) {
				shell.setSize(shellSize.x, shellSize.y - advancedGroupHeight);
				advancedGroupHeight = 0;
			}

		} else {
			Point groupSize = advancedGroup.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);

			// PageContainerFillLayout.marginHeight is equal to 5
			int marginHeight = 5 + ((GridLayout) this.getLayout()).marginHeight;
			advancedGroupHeight = marginHeight + advancedGroup.getBounds().height;
			int toolkitDialogTrayWidth = 0;
			// PageContainerFillLayout.marginWidth is equal to 5
			int marginWeight = 2 * 5 + ((GridLayout) advancedGroup.getLayout()).marginWidth + 2 * ((GridLayout) this.getLayout()).marginWidth;
			if (manager.getDialog().getTray() != null && manager.getDialog().getTray() instanceof ToolkitDialogTray) {
				ToolkitDialogTray toolkitDialogTray = (ToolkitDialogTray) manager.getDialog().getTray();
				// The margin of the leftSeparator and rightSeparator labels of
				// TrayDialog are both equal to 4
				toolkitDialogTrayWidth = toolkitDialogTray.getWidth() + 2 * 4;
			}
			advancedGroupWidth = groupSize.x + marginWeight + toolkitDialogTrayWidth;
			if (shellSize.x < advancedGroupWidth) {
				if (this.getBounds().height - advancedGroupHeight < advancedGroup.getBounds().y) {
					advancedGroupHeight = advancedGroup.getBounds().y + advancedGroupHeight - this.getBounds().height;
					shell.setSize(advancedGroupWidth, shellSize.y + advancedGroupHeight);
				} else {
					advancedGroupHeight = 0;
					shell.setSize(advancedGroupWidth, shellSize.y);
				}
			} else {
				if (this.getBounds().height - advancedGroupHeight < advancedGroup.getBounds().y) {
					advancedGroupHeight =advancedGroup.getBounds().y + advancedGroupHeight - this.getBounds().height;
					shell.setSize(shellSize.x, shellSize.y + advancedGroupHeight);
				} else {
					advancedGroupHeight = 0;
					shell.setSize(shellSize.x, shellSize.y);
				}
			}
		}
		composite.layout();
	}

}
