package com.sodius.rhapsody.toolkit.ui;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

import com.sodius.mdw.core.CoreException;
import com.sodius.rhapsody.toolkit.properties.RhapsodyXMIToolkitParameters;
import com.sodius.rhapsody.toolkit.properties.UMLVersionKind;
import com.sodius.rhapsody.toolkit.util.thread.RhapsodyAPIExec;
import com.telelogic.rhapsody.core.IRPProfile;
import com.telelogic.rhapsody.core.IRPProject;

public class ExportUMLFormatSelectionControl extends AdvancedBlocManager {

    private Map<UMLVersionKind, Button> umlChoices;
    private static String xmi10 = "XMI 1.0";
    private static String xmi11 = "XMI 1.1";
    private static String xmi12 = "XMI 1.2";
    private static String xmi21 = "XMI 2.1";
    private static Boolean isSysMLModel = null;
    private static Button sysmlB = null;
    private static Button exportRhapsodyProfileB = null;
    private boolean wasInitialized = false;

    public ExportUMLFormatSelectionControl(Composite parent, ExportUMLFormatSelectionWizardPage page, ToolkitUIManager manager, String titleLabe) {
        super(parent, page, manager, titleLabe);
        this.setLayout(new GridLayout(1, false));
        this.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        title = new Label(this, SWT.NONE);
        title.setText(titleLabel);

        createContent();
    }

    @Override
    protected void createFields() {

        rootComposite = new Composite(this, SWT.RADIO);
        rootComposite.setLayout(new GridLayout(2, false));
        rootComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        umlChoices = new HashMap<UMLVersionKind, Button>();

        Button uml21B = new Button(rootComposite, SWT.RADIO);
        uml21B.setText(UMLVersionKind.uml21.name);
        umlChoices.put(UMLVersionKind.uml21, uml21B);

        // Empty label
        new Label(rootComposite, SWT.NONE);

        Button uml22B = new Button(rootComposite, SWT.RADIO);
        uml22B.setText(UMLVersionKind.uml22.name);
        umlChoices.put(UMLVersionKind.uml22, uml22B);

        new Label(rootComposite, SWT.NONE);

        Button uml23B = new Button(rootComposite, SWT.RADIO);
        uml23B.setText(UMLVersionKind.uml23.name);
        umlChoices.put(UMLVersionKind.uml23, uml23B);

        new Label(rootComposite, SWT.NONE);
        Button uml241B = new Button(rootComposite, SWT.RADIO);
        uml241B.setText(UMLVersionKind.uml241.name);
        umlChoices.put(UMLVersionKind.uml241, uml241B);

        new Label(rootComposite, SWT.NONE);
        sysmlB = createCheckButton(rootComposite, "Embed SysML Profile", RhapsodyXMIToolkitParameters.KEY_EXPORT_EMBEDDED_SYSML_PROFILE, true);
        sysmlB.addSelectionListener(new SelectionListener() {

            public void widgetSelected(SelectionEvent e) {
                manager.getToolkit().getRhapsodyParameters().setExportEmbeddedSysMLProfile(sysmlB.getSelection());
            }

            public void widgetDefaultSelected(SelectionEvent e) {
            }

        });
        sysmlB.setVisible(false);

        // Export rhapsody profiles
        new Label(rootComposite, SWT.NONE);
        exportRhapsodyProfileB = createCheckButton(rootComposite, "Export Rhapsody Profiles",
                RhapsodyXMIToolkitParameters.KEY_EXPORT_WITH_RHAPSODY_EXTENSION, true);
        exportRhapsodyProfileB.setToolTipText("If selected, the exported XMI will contain an additional Model for roundtrip");
        exportRhapsodyProfileB.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                manager.getToolkit().getRhapsodyParameters().setExportRhapsodyExtension(exportRhapsodyProfileB.getSelection());
            }

            public void widgetDefaultSelected(SelectionEvent e) {
            }
        });

        // Empty label
        new Label(rootComposite, SWT.NONE);

        umlChoices.get(UMLVersionKind.uml21).setSelection(true);
    }

    protected void setDefault() {
        wasInitialized = true;
        if (this.manager.getToolkit().getParameters().isCommandLineProperty(RhapsodyXMIToolkitParameters.KEY_FORMAT)) {
            UMLVersionKind kind = this.manager.getToolkit().getParameters().getFormat(
                    this.manager.getToolkit().getParameters().getCommandLineProperties().getProperty(RhapsodyXMIToolkitParameters.KEY_FORMAT));
            for (UMLVersionKind choiceKind : umlChoices.keySet()) {
                Button choiceButton = umlChoices.get(choiceKind);
                if (choiceKind != kind) {
                    choiceButton.setSelection(false);
                } else {
                    choiceButton.setSelection(true);
                    choiceButton.notifyListeners(SWT.Selection, new Event());
                }

            }
            if ((sysmlB != null)
                    && this.manager.getToolkit().getParameters().isCommandLineProperty(RhapsodyXMIToolkitParameters.KEY_EXPORT_EMBEDDED_SYSML_PROFILE)
                    && isSysMLModel()) {
                sysmlB.setVisible(true);
                sysmlB.setEnabled(true);
                sysmlB.setSelection(
                        this.manager.getToolkit().getParameters().getProperty(RhapsodyXMIToolkitParameters.KEY_EXPORT_EMBEDDED_SYSML_PROFILE, false));
                sysmlB.notifyListeners(SWT.Selection, new Event());
            }
            if (this.manager.getToolkit().getParameters().isCommandLineProperty(RhapsodyXMIToolkitParameters.KEY_EXPORT_WITH_RHAPSODY_EXTENSION)) {

                exportRhapsodyProfileB.setEnabled(true);
                exportRhapsodyProfileB.setSelection(
                        this.manager.getToolkit().getParameters().getProperty(RhapsodyXMIToolkitParameters.KEY_EXPORT_WITH_RHAPSODY_EXTENSION,
                                RhapsodyXMIToolkitParameters.DEFAULT_EXPORT_WITH_RHAPSODY_EXTENSION));
                exportRhapsodyProfileB.notifyListeners(SWT.Selection, new Event());
            }

        } else if (isSysMLModel()) {
            sysmlB.setVisible(true);
            umlChoices.get(UMLVersionKind.uml21).setSelection(false);
            umlChoices.get(UMLVersionKind.uml23).setSelection(true);
            umlChoices.get(UMLVersionKind.uml23).notifyListeners(SWT.Selection, new Event());

            sysmlB.setEnabled(true);
            sysmlB.setSelection(false);
            sysmlB.notifyListeners(SWT.Selection, new Event());
        } else {
            umlChoices.get(UMLVersionKind.uml21).setSelection(true);
            umlChoices.get(UMLVersionKind.uml21).notifyListeners(SWT.Selection, new Event());
        }
    }

    protected boolean isUMLEAEnabled() {
        return manager.getToolkit().getParameters().getEnableUMLEA();
    }

    protected boolean isSysMLModel() {
        if (isSysMLModel == null) {
            final boolean[] result = new boolean[] { false };
            try {
                RhapsodyAPIExec.synchExec(new Runnable() {

                    public void run() {
                        try {
                            IRPProject project = manager.getToolkit().getEngine().getActiveProject();
                            if (project != null) {
                                IRPProfile sysMLProfile = (IRPProfile) project.findElementsByFullName("SysML", "Profile");
                                if (sysMLProfile != null) {
                                    result[0] = true;
                                }
                            }

                        } catch (CoreException e) {
                            result[0] = false;
                        }

                    }
                });
            } catch (Exception e) {
                result[0] = false;
            }

            isSysMLModel = result[0];
        }
        return isSysMLModel;
    }

    @Override
    protected void registerListener() {
        for (UMLVersionKind field : umlChoices.keySet()) {
            Button button = umlChoices.get(field);
            button.addSelectionListener(listener);
            button.addHelpListener(manager.getHelpListener());
        }
    }

    @Override
    protected void createAndResetListener() {
        listener = new SelectionListener() {

            public void widgetDefaultSelected(SelectionEvent e) {

            }

            public void widgetSelected(SelectionEvent e) {
                UMLVersionKind format = getSelectedMetamodelKind();
                if (format != null) {
                    if (page instanceof ExportUMLFormatSelectionWizardPage) {
                        ((ExportUMLFormatSelectionWizardPage) page).setIsUML21Node(true);
                    }
                }
                page.setParameters();

                if (umlChoices.containsValue(e.getSource()) && (advanced != null)) {
                    if ((advancedGroup != null) && advancedGroup.isVisible()) {
                        clearAdvancedProperties();
                        handleAdvancedButtonSelect();
                    }
                    clearAdvancedButton();

                } else if ((advanced != null) && (e.getSource() == advanced)) {
                    if (advancedGroup == null) {
                        createAdvancedProperties();
                    } else if (!advancedGroup.isVisible()) {
                        showAdvancedProperties();
                    } else {
                        clearAdvancedProperties();
                    }
                    handleAdvancedButtonSelect();
                } else if (isSysMLModel()) {
                    if ((e.getSource() == umlChoices.get(UMLVersionKind.uml21)) || (e.getSource() == umlChoices.get(UMLVersionKind.uml22))) {
                        sysmlB.setEnabled(true);
                        sysmlB.setSelection(true);
                        sysmlB.notifyListeners(SWT.Selection, new Event());
                    } else if (e.getSource() == umlChoices.get(UMLVersionKind.uml23)) {
                        sysmlB.setEnabled(true);
                        sysmlB.setSelection(false);
                        sysmlB.notifyListeners(SWT.Selection, new Event());
                    }
                }

            }
        };
    }

    @Override
    protected void createAdvancedProperties() {

        advanced.setText("<< Advanced");

        advancedGroup = new Group(this, SWT.NONE);
        advancedGroup.setText("Advanced options");
        advancedGroup.setLayout(new GridLayout(2, false));
        advancedGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        SelectionListener advancedListener = new SelectionListener() {

            public void widgetSelected(SelectionEvent e) {
                if (page instanceof ExportUMLFormatSelectionWizardPage) {
                    ((ExportUMLFormatSelectionWizardPage) page).setAdvancedProperties(manager.getToolkit().getRhapsodyParameters());
                }

            }

            public void widgetDefaultSelected(SelectionEvent e) {
            }

        };

        createCheckButton(advancedGroup, "Methods bodies", RhapsodyXMIToolkitParameters.KEY_EXPORT_METHODS,
                RhapsodyXMIToolkitParameters.DEFAULT_EXPORT_METHODS).addSelectionListener(advancedListener);
        createCheckButton(advancedGroup, "Tags", RhapsodyXMIToolkitParameters.KEY_EXPORT_TAGS, RhapsodyXMIToolkitParameters.DEFAULT_EXPORT_TAGS)
                .addSelectionListener(advancedListener);
        createCheckButton(advancedGroup, "Statecharts and activity diagrams", RhapsodyXMIToolkitParameters.KEY_EXPORT_STATEMACHINES,
                RhapsodyXMIToolkitParameters.DEFAULT_EXPORT_STATEMACHINES).addSelectionListener(advancedListener);
        createCheckButton(advancedGroup, "Annotations (Comments, Requirements, Constraints)", RhapsodyXMIToolkitParameters.KEY_EXPORT_ANNOTATIONS,
                RhapsodyXMIToolkitParameters.DEFAULT_EXPORT_ANNOTATIONS).addSelectionListener(advancedListener);
        createCheckButton(advancedGroup, "Collaborations and sequences diagrams", RhapsodyXMIToolkitParameters.KEY_EXPORT_COLLABORATIONS,
                RhapsodyXMIToolkitParameters.DEFAULT_EXPORT_COLLABORATIONS).addSelectionListener(advancedListener);
        createCheckButton(advancedGroup, "Description (HTML Format)", RhapsodyXMIToolkitParameters.KEY_EXPORT_DESCRIPTION_HTML,
                RhapsodyXMIToolkitParameters.DEFAULT_EXPORT_DESCRIPTION_HTML).addSelectionListener(advancedListener);
        createCheckButton(advancedGroup, "Deployment diagrams", RhapsodyXMIToolkitParameters.KEY_EXPORT_DEPLOYMENTS,
                RhapsodyXMIToolkitParameters.DEFAULT_EXPORT_DEPLOYMENTS).addSelectionListener(advancedListener);
        createCheckButton(advancedGroup, "Description (RTF Format)", RhapsodyXMIToolkitParameters.KEY_EXPORT_DESCRIPTION_RTF,
                RhapsodyXMIToolkitParameters.DEFAULT_EXPORT_DESCRIPTION_RTF).addSelectionListener(advancedListener);
        createCheckButton(advancedGroup, "Properties", RhapsodyXMIToolkitParameters.KEY_EXPORT_PROPERTIES,
                RhapsodyXMIToolkitParameters.DEFAULT_EXPORT_PROPERTIES).addSelectionListener(advancedListener);

        this.layout();

        // Register HelpListener on check buttons
        for (String propertyButtonName : properties.keySet()) {
            Button propertyButton = properties.get(propertyButtonName);
            propertyButton.addHelpListener(manager.getHelpListener());
        }
    }

    @Override
    public boolean getAdvancedPropertyValue(String property) {
        if (properties.get(property) != null) {
            Button pButton = properties.get(property);
            return pButton.getSelection();
        } else {
            return false;
        }
    }

    public UMLVersionKind getSelectedMetamodelKind() {
        for (UMLVersionKind field : umlChoices.keySet()) {
            if (umlChoices.get(field).getSelection()) {
                return field;
            }
        }
        return null;
    }

    public String getSelectedXMIWriter() {
        UMLVersionKind metamodelKind = getSelectedMetamodelKind();
        if (metamodelKind.equals(UMLVersionKind.uml21) || metamodelKind.equals(UMLVersionKind.uml22) || metamodelKind.equals(UMLVersionKind.uml23)) {
            return xmi21;
        } else {
            return null;
        }
    }
}
