package com.sodius.rhapsody.toolkit.ui;

import java.lang.reflect.InvocationTargetException;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.IPageChangedListener;
import org.eclipse.jface.dialogs.PageChangedEvent;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.util.SafeRunnable;
import org.eclipse.swt.widgets.Composite;

import com.sodius.mdw.core.CoreException;
import com.sodius.rhapsody.toolkit.properties.RhapsodyXMIToolkitParameters;
import com.sodius.rhapsody.toolkit.util.thread.RhapsodyAPIExec;

public class ExportUnitsSelectionWizardPage extends ToolkitWizardSelectionPage implements IPageChangedListener{

	private UnitsSelectionControl unitsSelection;
	ExportSaveFileWizardNode saveFileWizardNode;
	ExportSaveDirectoryWizardNode saveDirectoryWizardNode;
	private static String titleLabel = "Please select units to export:";
	private String lastModifiedTime=null;

	ExportUnitsSelectionWizardPage(ToolkitUIManager manager, String pageTitle) {
		super(manager, "ExportUnitsSelectionWizardPage", pageTitle, manager.getImageRegistry().getDescriptor(manager.getIconFileName()));
		setDescription(ExportWizard.DESCRIPTION);
	}

	public void createControl(Composite parent) {
		unitsSelection = new UnitsSelectionControl(parent, this, this.manager, titleLabel);
		setControl(unitsSelection);
		saveFileWizardNode = new ExportSaveFileWizardNode(manager);
		saveDirectoryWizardNode = new ExportSaveDirectoryWizardNode(manager);
		setSaveAsFile(!unitsSelection.isExportInSeparateFiles());
		updateTreeViewer();
		manageAutomaticPageComplete();
	}

	public void setParameters() {
		RhapsodyXMIToolkitParameters parameters = manager.getToolkit().getParameters();
		parameters.setExportInSeparateFiles(unitsSelection.isExportInSeparateFiles()) ;
		setSaveAsFile(!unitsSelection.isExportInSeparateFiles());
			
		List<Object> selectedElement = unitsSelection.getSelectedElement();
		selectedElement.addAll(unitsSelection.getGrayedElementContent());
		parameters.setSelectedUnitsToExport(selectedElement);
		parameters.setAllUnitsAreSelected(unitsSelection.getAllElementsAreSelected());
	}

	void setSaveAsFile(boolean set)
	{
		if(set)
			setSelectedNode(saveFileWizardNode);
		else
			setSelectedNode(saveDirectoryWizardNode);
			
	}
	
	private boolean isOutOfDate()
	{
		//return true if its current model is not the model in parameters
		return(!getRhapsodyModifiedTime().equals(lastModifiedTime));
	}
	
	private String getRhapsodyModifiedTime()
	{
		final String[] result = new String[1];
		try {
			RhapsodyAPIExec.synchExec(new Runnable() {
			
				public void run() {
					result[0]=manager.getToolkit().getEngine().getApplication().activeProject().getLastModifiedTime();
				}
			});
		} catch (CoreException e) {
			result[0]="";
			e.printStackTrace();
		}
		return result[0];
	}
	
	private void updateTreeViewer() {
		if(unitsSelection!=null && unitsSelection.getUnitsViewer()!=null)
		{
			lastModifiedTime=getRhapsodyModifiedTime();
			SafeRunnable.run(new RhapsodySafeRunnable(getShell()) {
			
				public void run() throws Exception {
					getWizard().getContainer().run(true, true, new IRunnableWithProgress() {
						
						public void run(IProgressMonitor monitor) throws InvocationTargetException,
								InterruptedException {
							monitor.beginTask("Refreshing Rhapsody Model...", -1);
							
							manager.getDisplay().syncExec(new Runnable() {
							
								public void run() {
									UnitsTreeViewer unitsViewer = unitsSelection.getUnitsViewer();
									unitsSelection.clearHiddenUnits();
									unitsViewer.setInput(manager.getToolkit().getEngine().getApplication());
									unitsViewer.expandAll();
									unitsViewer.collapseAll();
									unitsViewer.expandToLevel(2);
									unitsViewer.setAllChecked(true);
								}
							
							});
							monitor.done();
						}
					
					});
				}
			
			});
		}
		
	}
	
	
	public void pageChanged(PageChangedEvent event) {
        if (event.getSelectedPage() == this && isOutOfDate()) {
            //check if it needs to be updated
        	updateTreeViewer();
        }
    }

	public boolean isCommandLineFilled() {
		return this.manager.getToolkit().getParameters().isCommandLineProperty(RhapsodyXMIToolkitParameters.KEY_UNIT_SELECTION) &&
			this.manager.getToolkit().getParameters().isCommandLineProperty(RhapsodyXMIToolkitParameters.KEY_EXPORT_SEPARATE_FILE);
	}
	
	
}
