package com.sodius.rhapsody.toolkit.ui;

import java.io.File;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

public class HelpBrowser extends HelpSection{

	private int defaultWidth = 300;
	SelectionListener listener;
	String XMI_TOOLKIT_CHANGES = "XMI Toolkit changes";
	String XMI_TOOLKIT_USER_GUIDE = "XMI Toolkit user guide";
	String XMI_TOOLKIT_MAPPING_RULES = "XMI Toolkit mapping rules";
	String TAU_IMPORTER_PROCESS = "Tau importer process";
	String TAU_IMPORTER_MAPPING_RULES = "Tau importer mapping rules";

	String XMI_TOOLKIT_CHANGES_FILE = "XMI_Toolkit_Changes.pdf";
	String XMI_TOOLKIT_USER_GUIDE_FILE = "XMI_Toolkit_User_Guide.pdf";
	String XMI_TOOLKIT_MAPPING_RULES_FILE = "XMI_Toolkit_Mapping_Rules_Overview.pdf";
	String TAU_IMPORTER_PROCESS_FILE = "Rhapsody_Tau_Integration.pdf";
	String TAU_IMPORTER_MAPPING_RULES_FILE = "TauToRhapsodyDiagramsMapping.pdf";

	Map<Link, String> links = new HashMap<Link, String>();
	private ToolkitUIManager manager;
	private Composite helpComposite;
	private File docFolder;
	private Image normal;
	private Image hover;
	private IContributionItem closeAction;

	HelpBrowser(File file, ToolkitUIManager manager, Composite parent, int style) {
		super(parent, style);
		this.manager = manager;
		this.docFolder = file;
		this.setLayout(new GridLayout(1, false));
		this.setLayoutData(new GridData(GridData.FILL_VERTICAL));
		this.setBackground(this.getDisplay().getSystemColor(SWT.COLOR_WHITE));
		this.setBounds(0, 0, defaultWidth, this.getBounds().height);

		createContents();
		createListener();
		registerListeber();

		this.layout(true, true);
		this.pack();

	}

	private void createContents() {
		GridLayout layout = new GridLayout(1, true);
		layout.marginHeight=0;
		layout.marginTop = 0;
		layout.marginBottom = 0;
		layout.marginLeft=0;
		layout.marginRight=0;
		layout.marginWidth=0;
		
		Composite toolBarComposite = new Composite(this, SWT.NONE);
		toolBarComposite.setLayout(layout);
		toolBarComposite.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END ));

		ToolBar toolBar = new ToolBar(toolBarComposite, SWT.FLAT);
		toolBar.setBackground(this.getBackground());
		ToolBarManager toolBarManager = new ToolBarManager(toolBar);
		createActions();
		toolBarManager.add(closeAction);
		toolBarManager.update(true);
		helpComposite = new Composite(this, SWT.NONE);
		helpComposite.setLayout(new GridLayout(1, true));
		helpComposite.setLayoutData(new GridData(GridData.FILL_BOTH));
		helpComposite.setBackground(this.getDisplay().getSystemColor(SWT.COLOR_WHITE));

		Label toolkitDoc = new Label(helpComposite, SWT.NONE);
		toolkitDoc.setText("XMI Toolkit Documentation");
		toolkitDoc.setBackground(this.getDisplay().getSystemColor(SWT.COLOR_WHITE));
		FontData[] fontData = toolkitDoc.getFont().getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(10);
			fontData[i].setStyle(SWT.BOLD);
		}
		Font newFont = new Font(this.getDisplay(), fontData);
		toolkitDoc.setFont(newFont);

		links.put(createLink(XMI_TOOLKIT_CHANGES), XMI_TOOLKIT_CHANGES_FILE);
		links.put(createLink(XMI_TOOLKIT_USER_GUIDE), XMI_TOOLKIT_USER_GUIDE_FILE);
		links.put(createLink(XMI_TOOLKIT_MAPPING_RULES), XMI_TOOLKIT_MAPPING_RULES_FILE);
		Label tauRhpDoc = new Label(helpComposite, SWT.NONE);
		tauRhpDoc.setText("Tau - Rhapsody Integration Documentation");
		tauRhpDoc.setBackground(this.getDisplay().getSystemColor(SWT.COLOR_WHITE));
		tauRhpDoc.setFont(newFont);
		links.put(createLink(TAU_IMPORTER_PROCESS), TAU_IMPORTER_PROCESS_FILE);
		links.put(createLink(TAU_IMPORTER_MAPPING_RULES), TAU_IMPORTER_MAPPING_RULES_FILE);

		Label copyrightLabel = new Label(this, SWT.NONE);
		copyrightLabel.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		copyrightLabel.setText(" Sodius SAS, "+new GregorianCalendar().get(Calendar.YEAR));
		copyrightLabel.setBackground(this.getDisplay().getSystemColor(SWT.COLOR_WHITE));
		
		String version = manager.getToolkit().getParameters().getToolkitRelease();
		Label versionLabel = new Label(this, SWT.NONE);
		versionLabel.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		versionLabel.setText("version " + version);
		versionLabel.setBackground(this.getDisplay().getSystemColor(SWT.COLOR_WHITE));
	}

	private void createListener() {
		listener = new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {
				// TODO Auto-generated method stub

			}

			public void widgetSelected(SelectionEvent e) {
				if (links.containsKey(e.getSource())) {
					openLink((Link) e.getSource());
				}
			}

		};
	}

	private void registerListeber() {
		for (Iterator<Link> iterator = links.keySet().iterator(); iterator.hasNext();) {
			Link link = (Link) iterator.next();
			link.addSelectionListener(listener);
		}

	}

	private Link createLink(String name) {
		Link link = new Link(helpComposite, SWT.NONE);
		link.setText("<A>" + name + "</A>");
		link.setBackground(this.getDisplay().getSystemColor(SWT.COLOR_WHITE));
		FontData[] fontData = link.getFont().getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(10);
		}
		Font newFont = new Font(this.getDisplay(), fontData);
		link.setFont(newFont);
		return link;
	}

	private void openLink(Link link) {
		try {
			File file = new File(docFolder, links.get(link));
			BrowserControl.displayURL(file.toURI().toString());
		} catch (Exception e1) {
			// nothing happen
		}

	}

	public int getwidth() {
		return this.getBounds().width;
	}

	private void createImages() {
		Display display = Display.getCurrent();
		int[] shape = new int[] { 3, 3, 5, 3, 7, 5, 8, 5, 10, 3, 12, 3, 12, 5, 10, 7, 10, 8, 12, 10, 12, 12, 10, 12, 8, 10, 7, 10, 5, 12, 3, 12, 3, 10, 5, 8,
				5, 7, 3, 5 };

		/*
		 * Use magenta as transparency color since it is used infrequently.
		 */
		Color border = display.getSystemColor(SWT.COLOR_WIDGET_DARK_SHADOW);
		  Color background = display.getSystemColor(SWT.COLOR_LIST_BACKGROUND);
		  Color backgroundHot = new Color(display, new RGB(252, 160, 160));
		  Color transparent = display.getSystemColor(SWT.COLOR_MAGENTA);

		  PaletteData palette = new PaletteData(new RGB[] { transparent.getRGB(), border.getRGB(), background.getRGB(), backgroundHot.getRGB() });
		  ImageData data = new ImageData(16, 16, 8, palette);
		  data.transparentPixel = 0;

		  normal = new Image(display, data);
		  normal.setBackground(transparent);
		  GC gc = new GC(normal);
		  gc.setBackground(background);
		  gc.fillPolygon(shape);
		  gc.setForeground(border);
		  gc.drawPolygon(shape);
		  gc.dispose();

		  hover = new Image(display, data);
		  hover.setBackground(transparent);
		  gc = new GC(hover);
		  gc.setBackground(backgroundHot);
		  gc.fillPolygon(shape);
		  gc.setForeground(border);
		  gc.drawPolygon(shape);
		  gc.dispose();
		  
		backgroundHot.dispose();
	}

	private void createActions() {
		createImages();
		closeAction = new ContributionItem() {
			public void fill(ToolBar parent, int index) {
				final ToolItem item = new ToolItem(parent, SWT.PUSH);
				item.setImage(normal);
				item.setHotImage(hover);
				item.setToolTipText("close");
				item.addListener(SWT.Selection, new Listener() {
					public void handleEvent(org.eclipse.swt.widgets.Event event) {
						// close the tray
						manager.getDialog().closeTray();
					}
				});
			}
		};
	}

}
