package com.sodius.rhapsody.toolkit.ui;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.IPageChangedListener;
import org.eclipse.jface.dialogs.PageChangedEvent;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.util.SafeRunnable;
import org.eclipse.swt.widgets.Composite;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.model.Model;
import com.sodius.rhapsody.toolkit.properties.RhapsodyXMIToolkitParameters;

public class ImportUnitsSelectionWizardPage extends ToolkitWizardPage implements IPageChangedListener{

	private UnitsSelectionControl unitsSelection;
	private static String titleLabel = "Please select units to import:"; //@See RTC 124768  
	private static String pageDescription = "Select UML units to import in Rhapsody";
	private String lastXMIFileName=null;
	ImportUnitsSelectionWizardPage(ToolkitUIManager manager, String pageTitle) {
		super(manager, "ImportUnitsSelectionWizardPage", pageTitle, manager.getImageRegistry().getDescriptor(manager.getIconFileName()));
		setDescription(pageDescription);
	}

	public void createControl(Composite parent) {
		unitsSelection = new UnitsSelectionControl(parent, this, this.manager, titleLabel);
		setControl(unitsSelection);
		this.setPageComplete(true);
		updateTreeViewer();
	}

	public void setParameters() {
		RhapsodyXMIToolkitParameters parameters = manager.getToolkit().getParameters();
		parameters.setSelectedNamespacesToImport(unitsSelection.getSelectedElement());
		parameters.setAllUnitsAreSelected(unitsSelection.getAllElementsAreSelected());
	}

	private boolean isOutOfDate()
	{
		//return true if its current model is not the model in parameters
		if(manager.getXMIFilename().equals(lastXMIFileName))
			return false;
		return true;
	}
	
	private void updateTreeViewer() {
		if(unitsSelection!=null && unitsSelection.getUnitsViewer()!=null)
		{
			SafeRunnable.run(new RhapsodySafeRunnable(getShell()) {
			
				public void run() throws Exception {
					getWizard().getContainer().run(true, true, new IRunnableWithProgress() {
						
						public void run(IProgressMonitor monitor) throws InvocationTargetException,
								InterruptedException {
							monitor.beginTask("Loading UML Model...", -1);
							lastXMIFileName=manager.getXMIFilename();
							
							try {
								final Model model = manager.getImportModel();
								
								manager.getDisplay().syncExec(new Runnable() {
								
									public void run() {
										UnitsTreeViewer unitsViewer = unitsSelection.getUnitsViewer();
										unitsSelection.clearHiddenUnits();
										unitsViewer.setInput(model);
										unitsViewer.expandToLevel(1);
										unitsViewer.setAllChecked(true);
									}
								
								});
								monitor.done();
							}
							catch (CoreException e) {
								throw new InvocationTargetException(e);
							}
						}
					
					});
				}
			
			});
		}
		
	}
	
	
	public void pageChanged(PageChangedEvent event) {
        if (event.getSelectedPage() == this && isOutOfDate()) {
            //check if it needs to be updated
        	updateTreeViewer();
        }
    }

	public boolean isCommandLineFilled() {
		return (this.manager.getToolkit().getParameters().isCommandLineProperty(RhapsodyXMIToolkitParameters.KEY_UNIT_SELECTION));
	}
}
