package com.sodius.rhapsody.toolkit.ui;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.util.SafeRunnable;
import org.eclipse.swt.widgets.Shell;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.eval.EvaluationException;

public abstract class RhapsodySafeRunnable extends SafeRunnable {
    
    private Shell shell;

    public RhapsodySafeRunnable(Shell shell) {
        super();
        this.shell=shell;
    }
    
    
    private static MultiStatus createMultiStatus(String msg, Throwable t) {

        List<Status> childStatuses = new ArrayList<Status>();
        StackTraceElement[] stackTraces = Thread.currentThread().getStackTrace();

         for (StackTraceElement stackTrace: stackTraces) {
                Status status = new Status(IStatus.ERROR,
                        RhapsodySafeRunnable.class.getPackage().getName(), stackTrace.toString());
                childStatuses.add(status);
        }

        MultiStatus ms = new MultiStatus(RhapsodySafeRunnable.class.getPackage().getName(),
                        IStatus.ERROR, childStatuses.toArray(new Status[] {}),
                        t.toString(), t);
        return ms;
    }
    
    @Override
    public void handleException(Throwable e) {
        handleException(shell, e);
    }
    
    public static void handleException(Shell shell, Throwable e) {
        if(e instanceof InvocationTargetException){
            e=e.getCause();
        }
        if(e instanceof CoreException && e.getCause()!=null){
            e=e.getCause();
        }
        if(e instanceof EvaluationException && e.getCause()!=null){
            e=e.getCause();
        }
        if(e instanceof OutOfMemoryError){
            ErrorDialog.openError(shell, "Out of memory", "The model size requires more memory than allowed by current setup.\nPlease check section \"Managing huge models\" in XMIToolkit User Guide for increasing memory.", createMultiStatus("Message", e));   
        }
        else{
            ErrorDialog.openError(shell, "Unexpected error", "An unexpected error has been thrown", createMultiStatus("Message", e));
        }
    }
    
}
