package com.sodius.rhapsody.toolkit.ui;

import java.io.File;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class ShowDirectoryControl extends Composite {

	private ToolkitUIManager manager;
	private SaveDirectoryWizardPage page;
	private Label title;
	private Composite saveDirectoryComposite;
	private Text DirectoryName;

	private static String Area_errorNotDirectory = "This file is not a directory";

	ShowDirectoryControl(Composite parent, SaveDirectoryWizardPage page, ToolkitUIManager manager, String label) {
		super(parent, SWT.NONE);
		this.page = page;
		this.setLayout(new GridLayout(1, false));
		this.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		this.manager = manager;

		title = new Label(this, SWT.NONE);
		title.setText(label);

		saveDirectoryComposite = new Composite(this, SWT.NONE);
		saveDirectoryComposite.setLayout(new GridLayout(2, false));
		saveDirectoryComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		createContents();
	}

	public void createContents() {
		DirectoryName = new Text(saveDirectoryComposite, SWT.BORDER);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		DirectoryName.setLayoutData(data);
		DirectoryName.addHelpListener(manager.getHelpListener());
		Button save = new Button(saveDirectoryComposite, SWT.PUSH);
		save.setText("Browse");
		save.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				// User has selected to save a file
				DirectoryDialog dlg = new DirectoryDialog(ShowDirectoryControl.this.getShell(), SWT.SAVE);
				String fn = dlg.open();
				if (fn != null) {
					DirectoryName.setText(fn);
				}
			}
		});
		DirectoryName.addModifyListener(new ModifyListener() {

			public void modifyText(ModifyEvent e) {
				String directoryPath = DirectoryName.getText().trim();
				String message = computeURIErrorMessage(directoryPath);
				if (message == null && directoryPath.length() > 0) {
					if (!manager.isImportMode()) {
						manager.setDirectoryname(directoryPath);
						page.setPageComplete(true);
					}
				} else {
					if (!manager.isImportMode()) {

						page.setPageComplete(false);
					}
				}
				manager.getDialog().setErrorMessage(computeURIErrorMessage(directoryPath));

			}
		});
		save.addHelpListener(manager.getHelpListener());
		this.layout(true, true);
		this.pack();
	}

	protected String computeURIErrorMessage(String resolvedURI) {
		File file = new File(resolvedURI);

		// ! directory ?
		if (file.exists() && !file.isDirectory())
			return Area_errorNotDirectory;

		return null;
	}

}