package com.sodius.rhapsody.toolkit.ui;

import java.io.File;

import org.eclipse.jface.dialogs.IPageChangedListener;
import org.eclipse.jface.dialogs.PageChangedEvent;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.dialogs.TrayDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.jface.wizard.WizardSelectionPage;
import org.eclipse.swt.events.HelpEvent;
import org.eclipse.swt.events.HelpListener;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.model.Metamodel;
import com.sodius.mdw.core.model.Model;
import com.sodius.mdw.core.model.io.ConnectorDescriptor;
import com.sodius.mdw.metamodel.uml21.io.ea.Options;
import com.sodius.mdw.metamodel.uml21.io.ea.utils.InternalOptions;
import com.sodius.rhapsody.toolkit.engine.uml21.UML21ImportRunner;
import com.sodius.rhapsody.toolkit.properties.RhapsodyXMIToolkitParameters;
import com.sodius.rhapsody.toolkit.properties.UMLVersionKind;

public class ToolkitUIManager {

    private static final String icon_folder = "icons";
    public static final String RHP_FOLDER = "rhp";

    public static final String EXPORT = "export.png";
    public static final String IMPORT = "import.png";
    public static final String RHAPSODY = "rhp.png";
    public static final String CLOSE = "delete_edit_002.gif";
    public static final String CLOSE_BACKGROUND = "delete_obj.gif";

    public static final String RHP_ACTOR = "Actor.gif";
    public static final String RHP_ASSOCIATIONCLASS = "AssociationClass.gif";
    public static final String RHP_CLASS = "Class.gif";
    public static final String RHP_COLLABORATIONDIAGRAM = "CollaborationDiagram.gif";
    public static final String RHP_COMPONENT = "Component.gif";
    public static final String RHP_CONTROLLEDFILE = "ControlledFile.gif";
    public static final String RHP_EVENT = "Event.gif";
    public static final String RHP_EVENTRECEPTION = "EventReception.gif";
    public static final String RHP_FILE = "File.gif";
    public static final String RHP_FLOWCHART = "Flowchart.gif";
    public static final String RHP_FLOWITEM = "FlowItem.gif";
    public static final String RHP_INSTANCE = "Instance.gif";
    public static final String RHP_MODULE = "Module.gif";
    public static final String RHP_NODE = "Node.gif";
    public static final String RHP_OPERATION = "Operation.gif";
    public static final String RHP_PACKAGE = "Package.gif";
    public static final String RHP_PORT = "Port.gif";
    public static final String RHP_PROFILE = "Profile.gif";
    public static final String RHP_PROJECT = "Project.gif";
    public static final String RHP_RELATION = "Relation.gif";
    public static final String RHP_SEQUENCEDIAGRAM = "SequenceDiagram.gif"; // @See RTC 140913
    public static final String RHP_STATECHART = "Statechart.gif";
    public static final String RHP_STEREOTYPE = "Stereotype.gif";
    public static final String RHP_SYSMLPORT = "SysMLPort.gif";
    public static final String RHP_TYPE = "Type.gif";
    public static final String RHP_USECASE = "UseCase.gif";

    private final Display display;
    private ImageRegistry imageRegistry;
    private HelpListener helpListener;
    private final RhapsodyXMIToolkit toolkit;
    private ToolkitWizard wizard;
    private WizardDialog dialog;
    private String xmiFilename;
    private boolean wasLaunched = false;

    public ToolkitUIManager(RhapsodyXMIToolkit toolkit) {
        super();
        display = Display.getDefault();
        this.toolkit = toolkit;
        toolkit.setManager(this);
    }

    public void openImportWizard() {
        wizard = new ImportWizard(this);
        createDialog(wizard);
        if (!(System.getProperty("os.name").startsWith("Linux")) && getToolkit().getParameters().getHelpIsOpen()) {
            openTray();
        }
        while (wizard.isMainPageCommandLineFilled() && (wizard.getMainPage() instanceof WizardSelectionPage)) {
            if (((WizardSelectionPage) wizard.getMainPage()).getSelectedNode() != null) {
                IWizard next = ((WizardSelectionPage) wizard.getMainPage()).getSelectedNode().getWizard();
                if (next instanceof ToolkitWizard) {
                    wizard = (ToolkitWizard) next;
                    createDialog(wizard);
                } else {
                    break;
                }
            } else {
                break;
            }
        }
        openWizard();
    }

    public void openExportWizard() {
        wizard = new ExportWizard(this);
        createDialog(wizard);
        while (wizard.isMainPageCommandLineFilled() && (wizard.getMainPage() instanceof WizardSelectionPage)) {
            IWizard next = ((WizardSelectionPage) wizard.getMainPage()).getSelectedNode().getWizard();
            if (next instanceof ToolkitWizard) {
                wizard = (ToolkitWizard) next;
                createDialog(wizard);
            } else {
                break;
            }
        }
        openWizard();
    }

    public Display getDisplay() {
        return display;
    }

    public ImageRegistry getImageRegistry() {
        if (imageRegistry == null) {
            imageRegistry = new ImageRegistry(display);
            registerImage(EXPORT);
            registerImage(IMPORT);
            registerImage(RHAPSODY);
            registerImage(CLOSE);
            registerImage(CLOSE_BACKGROUND);
            registerRHPImage(RHP_ACTOR);
            registerRHPImage(RHP_ASSOCIATIONCLASS);
            registerRHPImage(RHP_CLASS);
            registerRHPImage(RHP_COLLABORATIONDIAGRAM);
            registerRHPImage(RHP_COMPONENT);
            registerRHPImage(RHP_CONTROLLEDFILE);
            registerRHPImage(RHP_EVENT);
            registerRHPImage(RHP_EVENTRECEPTION);
            registerRHPImage(RHP_FILE);
            registerRHPImage(RHP_FLOWCHART);
            registerRHPImage(RHP_FLOWITEM);
            registerRHPImage(RHP_INSTANCE);
            registerRHPImage(RHP_MODULE);
            registerRHPImage(RHP_NODE);
            registerRHPImage(RHP_OPERATION);
            registerRHPImage(RHP_PACKAGE);
            registerRHPImage(RHP_PORT);
            registerRHPImage(RHP_PROFILE);
            registerRHPImage(RHP_PROJECT);
            registerRHPImage(RHP_RELATION);
            registerRHPImage(RHP_SEQUENCEDIAGRAM);
            registerRHPImage(RHP_STATECHART);
            registerRHPImage(RHP_STEREOTYPE);
            registerRHPImage(RHP_SYSMLPORT);
            registerRHPImage(RHP_TYPE);
            registerRHPImage(RHP_USECASE);
        }
        return imageRegistry;
    }

    private void registerImage(String name) {

        // String file = icon_folder + File.separator + name;
        String file = "/" + icon_folder + "/" + name;
        // String path = new File(java.net.URI.create(ClassLocator.getSourceLocation(XMIToolkitPlugin.class).toString()).getPath()).getParentFile()
        // .getAbsolutePath();
        // String path = XMIToolkitPlugin.class.getResource(file).getFile();
        // path += File.separator + file;
        // try {
        imageRegistry.put(name, ImageDescriptor.createFromURL(ToolkitUIManager.class.getResource(file)));// new File(path).toURL()));
        // } catch (MalformedURLException e) {
        // // TODO Auto-generated catch block
        // e.printStackTrace();
        // }

    }

    private void registerRHPImage(String name) {
        // registerImage(RHP_FOLDER + File.separator + name);
        registerImage(RHP_FOLDER + "/" + name);
    }

    public RhapsodyXMIToolkit getToolkit() {
        return toolkit;
    }

    public boolean isImportMode() {
        return this.getToolkit().getParameters().isImportMode();
    }

    public IWizard getWizard() {
        return wizard;
    }

    public void setDialog(WizardDialog dialog) {
        this.dialog = dialog;
    }

    public HelpListener getHelpListener() {
        if (helpListener == null) {
            helpListener = new HelpListener() {

                public void helpRequested(HelpEvent e) {
                    if (dialog != null) {
                        if (dialog.getTray() == null) // tray closed => we open it
                        {
                            openTray();
                        } else // tray opened => we close it
                        {
                            closeTray();
                        }
                    }

                }
            };
        }
        return helpListener;
    }

    protected void closeTray() {
        dialog.closeTray();
        getToolkit().getParameters().setHelpIsOpen(false);
    }

    protected void openTray() {
        dialog.openTray(new ToolkitDialogTray(getHelpFilesFolder(), ToolkitUIManager.this));
        getToolkit().getParameters().setHelpIsOpen(true);
    }

    private File getHelpFilesFolder() {
        String htmlFiles = toolkit.getToolkitDirectory() + File.separator + "doc";
        File helpFolder = new File(htmlFiles);
        return helpFolder;
    }

    public void setHelpListener(HelpListener helpListener) {
        this.helpListener = helpListener;
    }

    public WizardDialog getDialog() {
        return dialog;
    }

    public String getXMIFilename() {
        return xmiFilename;
    }

    public void setXmiFilename(String xmiFilename) {
        this.xmiFilename = xmiFilename;
    }

    public void setDirectoryname(String directoryname) {
        this.xmiFilename = directoryname;
    }

    protected WizardDialog createDialog(ToolkitWizard wizard) {
        // create dialog
        TrayDialog.setDialogHelpAvailable(true);
        dialog = new WizardDialog(display.getActiveShell(), wizard);
        dialog.setTitle(wizard.getWindowTitle());
        dialog.create();
        dialog.buttonBar.addHelpListener(this.getHelpListener());

        dialog.addPageChangedListener(new IPageChangedListener() {

            public void pageChanged(PageChangedEvent event) {
                if (event.getSelectedPage() instanceof IPageChangedListener) {
                    ((IPageChangedListener) event.getSelectedPage()).pageChanged(event);
                }
            }

        });

        return dialog;
    }

    protected void openWizard() {
        Shell parentShell = display.getActiveShell();
        if (parentShell == null) {
            parentShell = new Shell(display);
            parentShell.setImage(getImageRegistry().get(ToolkitUIManager.RHAPSODY));
        }

        if (parentShell.getImage() != null) {
            dialog.getShell().setImage(parentShell.getImage());
        }

        dialog.open();
        display.dispose();
    }

    public boolean performFinish() {
        if (wasLaunched) {
            return true;
        }
        wasLaunched = true;
        final Display display = Display.getDefault();
        final ProgressMonitorDialog monitorDialog = new ProgressMonitorDialog(display.getActiveShell());
        try {
            toolkit.actionProceed(monitorDialog);
        } finally {
            cleanAtEnd();
            return true;
        }
    }

    protected void cleanAtEnd() {
        toolkit.cleanAtEnd();
    }

    public boolean performCancel() {
        cleanAtEnd();
        return true;
    }

    public String getIconFileName() {
        if (getToolkit().getParameters().isImportMode()) {
            return IMPORT;
        } else {
            return EXPORT;
        }
    }

    public Model getImportModel() throws CoreException {
        // TODO: the manager or the parameters should offer only one public getter for accessing the model.
        // should check internaly if the already read import model is up 2 date or not.
        // Currently the import model is buggy and you risk to import a model which is the last "unit treeaged" one.
    	ConnectorDescriptor selectedReader = UML21ImportRunner.getUMLModelConnector(toolkit.getParameters());

    	Model model = toolkit.getParameters().getMDWorkbench().getMetamodelManager()
    			.getMetamodel("uml21").readModel(selectedReader != null ? selectedReader.getName() : "XMI", toolkit.getParameters().getXMIFilename());
        getToolkit().getParameters().setAlreadyReadImportModel(model);
        return model;
    }
}
