package com.sodius.rhapsody.toolkit.ui;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardPage;

public abstract class ToolkitWizard extends Wizard {

	ToolkitUIManager manager;
    protected WizardPage mainPage;
	
	public ToolkitWizard(ToolkitUIManager manager) {
		super();
		this.manager = manager;
		setNeedsProgressMonitor(true);
		setForcePreviousAndNextButtons(true);
	}

	public ToolkitUIManager getManager() {
		return manager;
	}

	public boolean performFinish() {
		return manager.performFinish();
	}
	
	@Override
	public boolean performCancel() {
		return manager.performCancel();
	}

	public final boolean isMainPageCommandLineFilled()
	{
		if(mainPage instanceof IToolkitParameterSetterWizardPage)
			return ((IToolkitParameterSetterWizardPage)mainPage).isCommandLineFilled();
		return false;
	}
	
	public final boolean isRecursiveCommandLineFilled()
	{
		if(this.getPageCount()==0)
			return false;
		boolean isRecursiveCommandLineFilled=true;
		for (IWizardPage page : this.getPages()) {
			if(page instanceof IToolkitParameterSetterWizardPage)
			{
				isRecursiveCommandLineFilled = ((IToolkitParameterSetterWizardPage)mainPage).isRecursiveCommandLineFilled();
			}
			else
				isRecursiveCommandLineFilled = false;
			if(!isRecursiveCommandLineFilled)
				break;
		}
		return isRecursiveCommandLineFilled;
	}
	
	public WizardPage getMainPage() {
		return mainPage;
	}

	@Override
	public IWizardPage getPage(String name) {
		getPagesReady();
		return super.getPage(name);
	}
	
	@Override
	public int getPageCount() {
		getPagesReady();
		return super.getPageCount();
	}
	
	private void getPagesReady()
	{
		if(super.getPageCount()==0)
			this.addPages();
	}
	
	protected boolean isPagesReady()
	{
		return super.getPageCount()>0;
	}
}
