package com.sodius.rhapsody.toolkit.ui;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import com.sodius.rhapsody.toolkit.properties.RhapsodyXMIToolkitParameters;

public class UnitsSelectionControl extends Composite {

	private Button separateFileSelection;
	private Label title;
	private Composite unitsSelectionComposite;
	private IToolkitParameterSetterWizardPage page;
	private ToolkitUIManager manager;
	private UnitsTreeViewer unitsViewer;
	/*
	 * The TreeItem does not show all elements. Some of them are hidden
	 * Those lists allow the possibility to register hidden elements
	 */
	private List<Object> hiddenUnitsAlwaysSelected = new ArrayList<Object>();
	private Map<Object,List<Object>> hiddenUnitsSelectedWithParents = new HashMap<Object, List<Object>>(); //parent, list of selected units

	public UnitsSelectionControl(Composite parent, IToolkitParameterSetterWizardPage unitsSelectionExportWizardPage, ToolkitUIManager manager, String titleLabel) {
		super(parent, SWT.NONE);
		this.page = unitsSelectionExportWizardPage;
		this.setLayout(new GridLayout(1, false));
		this.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		this.manager = manager;
		title = new Label(this, SWT.NONE);
		title.setText(titleLabel);

		unitsSelectionComposite = new Composite(this, SWT.NONE);
		unitsSelectionComposite.setLayout(new GridLayout(1, false));
		unitsSelectionComposite.setLayoutData(new GridData(GridData.FILL_BOTH));
		createContent();
		unitsSelectionComposite.layout();

		this.layout(true, true);
		pack();
	}

	private void createContent() {

		createFields();
		registerListener();
	}

	private void createFields() {

		// Create the tree viewer
		unitsViewer = new UnitsTreeViewer(unitsSelectionComposite);
		unitsViewer.getTree().setLayoutData(new GridData(GridData.FILL_BOTH));
		if (manager.isImportMode()) {
			unitsViewer.setContentProvider(new UnitsTreeImportContentProvider(this));
		}else{
			unitsViewer.setContentProvider(new UnitsTreeExportContentProvider(this));
		}
		unitsViewer.setLabelProvider(new UnitsTreeLabelProvider(manager));
		
		if (manager.isImportMode()){
			unitsViewer.expandToLevel(1);
		}else{
			unitsViewer.expandToLevel(2);
		}
		
		unitsViewer.setAllChecked(true);
		
		if (!manager.isImportMode()) {
			// Create separateFileSelection button
			separateFileSelection = new Button(this, SWT.CHECK);
			separateFileSelection.setText("Export in separate files");
			separateFileSelection.setSelection(manager.getToolkit().getParameters().getExportInSeparateFiles());
			separateFileSelection.addSelectionListener(new SelectionListener() {
			
				public void widgetSelected(SelectionEvent e) {
					page.setParameters();
				}
			
				public void widgetDefaultSelected(SelectionEvent e) {
					// TODO Auto-generated method stub
				}
			
			});
			if(manager.getToolkit().getParameters().isCommandLineProperty(RhapsodyXMIToolkitParameters.KEY_EXPORT_SEPARATE_FILE))
				separateFileSelection.setEnabled(false);
		}
	}

	private void registerListener() {
		if (!manager.isImportMode()) {
			separateFileSelection.addHelpListener(manager.getHelpListener());
			separateFileSelection.addSelectionListener(new SelectionListener() {
			
				public void widgetSelected(SelectionEvent e) {
					// TODO Auto-generated method stub
				}
			
				public void widgetDefaultSelected(SelectionEvent e) {
					if (isExportInSeparateFiles()) {
						if(page instanceof ExportUnitsSelectionWizardPage)
							((ExportUnitsSelectionWizardPage)page).setSaveAsFile(false);
					} else {
						if(page instanceof ExportUnitsSelectionWizardPage)
							((ExportUnitsSelectionWizardPage)page).setSaveAsFile(true);
					}
				}
			});
		}
		unitsViewer.addCheckStateListener(new ICheckStateListener() {
		
			public void checkStateChanged(CheckStateChangedEvent event) {
				page.setParameters();
			}
		});
		unitsSelectionComposite.addHelpListener(manager.getHelpListener());
	}

	public boolean isExportInSeparateFiles() {
		if (separateFileSelection != null)
			return separateFileSelection.getSelection();
		else
			return false;
	}

	public List<Object> getSelectedElement() {
		List<Object> returnList = new ArrayList<Object>(Arrays.asList(unitsViewer.getCheckedElements()));
		returnList.addAll(hiddenUnitsAlwaysSelected);
		List<Object> hiddenToAdd = new ArrayList<Object>();
		for (Object element : returnList) {
			if(hiddenUnitsSelectedWithParents.containsKey(element))
				hiddenToAdd.addAll(hiddenUnitsSelectedWithParents.get(element));
		}
		returnList.addAll(hiddenToAdd);
		return returnList;
	}
	
	public List<Object> getGrayedElementContent() {
		List<Object> returnList = new ArrayList<Object>(Arrays.asList(unitsViewer.getGrayedElements()));
		List<Object> hiddenToAdd = new ArrayList<Object>();
		for (Object element : returnList) {
			if(hiddenUnitsSelectedWithParents.containsKey(element))
				hiddenToAdd.addAll(hiddenUnitsSelectedWithParents.get(element));
		}
//		returnList.addAll(hiddenToAdd);
		return hiddenToAdd;
	}
	
	
	public boolean getAllElementsAreSelected() {
		return unitsViewer.getAllElementsAreSelected();
	}

	public UnitsTreeViewer getUnitsViewer() {
		return unitsViewer;
	}
/*
 * 	private List<Object> hiddenUnitsAlwaysSelected = new ArrayList<Object>();
	private Map<Object,Object> hiddenUnitsSelectedWithParents = new HashMap<Object, Object>();
 */	
	
	void addHiddenUnitsAlwaysSelected(Object element)
	{
		hiddenUnitsAlwaysSelected.add(element);
	}

	void addHiddenUnitsSelectedWithParents(Object parent, Object element)
	{
		List<Object> selectedUnits;
		if(hiddenUnitsSelectedWithParents.containsKey(parent))
			selectedUnits=hiddenUnitsSelectedWithParents.get(parent);
		else
			selectedUnits=new ArrayList<Object>();
		selectedUnits.add(element);
		if(selectedUnits.size()==1)
			hiddenUnitsSelectedWithParents.put(parent, selectedUnits);
	}
	
	void clearHiddenUnits()
	{
		hiddenUnitsAlwaysSelected.clear();
		hiddenUnitsSelectedWithParents.clear();
	}
	
}
