package com.sodius.rhapsody.toolkit.ui;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import com.sodius.mdw.core.CoreException;
import com.sodius.rhapsody.toolkit.util.RhapsodyUnitHandler;
import com.sodius.rhapsody.toolkit.util.thread.RhapsodyAPIExec;
import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.IRPPackage;
import com.telelogic.rhapsody.core.IRPProject;
import com.telelogic.rhapsody.core.IRPUnit;

public class UnitsTreeExportContentProvider implements ITreeContentProvider {

	private final UnitsSelectionControl control;
	
	UnitsTreeExportContentProvider(UnitsSelectionControl control) {
		super();
		this.control = control;
	}
	
	private Object[] getChildrenUnSecuredThread(Object parent, boolean registerHiddenUnits)
	{
		if (parent instanceof IRPApplication){
			return new Object[]{new RhapsodyUnitHandler(((IRPApplication)parent).activeProject())};
		}
		else if (parent instanceof IRPUnit) {
			List<RhapsodyUnitHandler> returnList = new ArrayList<RhapsodyUnitHandler>();

			for (IRPUnit subUnit : (List<IRPUnit>)((IRPUnit)parent).getNestedSaveUnits().toList()) {
				returnList.add(new RhapsodyUnitHandler(subUnit));
			}
			
			final List children = ((IRPUnit) parent).getNestedElements().toList();
			for (Object child : children) {
				if(child instanceof IRPModelElement && 
						(!(child instanceof IRPUnit) || ((IRPUnit)child).isSeparateSaveUnit()==0))
					control.addHiddenUnitsSelectedWithParents(new RhapsodyUnitHandler((IRPUnit)parent), new RhapsodyUnitHandler((IRPModelElement)child));		
			}
			children.clear();
			
			// Project case

			if (parent instanceof IRPProject) {
				//@See 124763 
				Set<IRPUnit> savedUnits = new HashSet<IRPUnit>((List<IRPUnit>)((IRPUnit)parent).getNestedSaveUnits().toList());
				
				// 1) Adding each profiles
				
				for (IRPUnit subUnit : (List<IRPUnit>)(((IRPProject) parent).getProfiles().toList())) {
					if(!savedUnits.contains(subUnit))
						returnList.add(new RhapsodyUnitHandler(subUnit));
				}
				// 2) Adding predefined types
				if(registerHiddenUnits)
				{
					IRPPackage pdt = (IRPPackage) ((IRPProject) parent).findElementsByFullName("PredefinedTypes", "Package");
	
					IRPPackage pdtJava = (IRPPackage) ((IRPProject) parent).findElementsByFullName("PredefinedTypesJava", "Package");
	
					IRPPackage pdtCpp = (IRPPackage) ((IRPProject) parent).findElementsByFullName("PredefinedTypesCpp", "Package");
	
					IRPPackage pdtC = (IRPPackage) ((IRPProject) parent).findElementsByFullName("PredefinedTypesC", "Package");
	
					IRPPackage pdtAda = (IRPPackage) ((IRPProject) parent).findElementsByFullName("PredefinedTypesAda", "Package");
	
					if (pdt != null)
						control.addHiddenUnitsAlwaysSelected(new RhapsodyUnitHandler(pdt));
						//returnList.add(new RhapsodyUnitTreeItem(pdt));
	
					if (pdtJava != null)
						control.addHiddenUnitsAlwaysSelected(new RhapsodyUnitHandler(pdtJava));
						//returnList.add(new RhapsodyUnitTreeItem(pdtJava));
	
					if (pdtCpp != null)
						control.addHiddenUnitsAlwaysSelected(new RhapsodyUnitHandler(pdtCpp));
						//returnList.add(new RhapsodyUnitTreeItem(pdtCpp));
	
					if (pdtC != null)
						control.addHiddenUnitsAlwaysSelected(new RhapsodyUnitHandler(pdtC));
						//returnList.add(new RhapsodyUnitTreeItem(pdtC));
	
					if (pdtAda != null)
						control.addHiddenUnitsAlwaysSelected(new RhapsodyUnitHandler(pdtAda));
						//returnList.add(new RhapsodyUnitTreeItem(pdtAda));
				}
			}
			return returnList.toArray();
		}
		return new Object[]{};
	}
	
	public Object[] getChildren(final Object parent, final boolean registerHiddenUnits) {
		
		final List<Object> result = new ArrayList<Object>();
		if(parent instanceof RhapsodyUnitHandler)
		{
			try {
				RhapsodyAPIExec.synchExec(new Runnable() {
				
					public void run() {
						
						result.addAll(Arrays.asList(getChildrenUnSecuredThread(((RhapsodyUnitHandler)parent).getIRPInterface(), registerHiddenUnits)));
					}
				});
			} catch (CoreException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return result.toArray();
	}
	
	public Object[] getChildren(final Object parent) {
		
		return getChildren(parent, true);
	}

	public Object getParent(Object element) {
		
		return null;
	}

	public boolean hasChildren(Object element) {
		return (element instanceof RhapsodyUnitHandler && (getChildren(element,false).length>0));
	}

	public Object[] getElements(Object inputElement) {
		if (inputElement instanceof IRPApplication) {
			final IRPApplication application = ((IRPApplication) inputElement);
			final RhapsodyUnitHandler[] project=new RhapsodyUnitHandler[1];
			try {
				RhapsodyAPIExec.synchExec(new Runnable() {
				
					public void run() {
						project[0] = new RhapsodyUnitHandler(application.activeProject());
					}
				
				});
			} catch (CoreException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return project;
		}
		return null;
	}

	public void dispose() {
	}

	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		// TODO Auto-generated method stub

	}

}
