package com.sodius.rhapsody.toolkit.ui;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import com.sodius.mdw.core.model.Model;
import com.sodius.mdw.metamodel.uml21.Association;
import com.sodius.mdw.metamodel.uml21.BehavioralFeature;
import com.sodius.mdw.metamodel.uml21.Element;
import com.sodius.mdw.metamodel.uml21.InformationItem;
import com.sodius.mdw.metamodel.uml21.InteractionOperand;
import com.sodius.mdw.metamodel.uml21.Namespace;
import com.sodius.mdw.metamodel.uml21.OpaqueBehavior;
import com.sodius.mdw.metamodel.uml21.Region;
import com.sodius.mdw.metamodel.uml21.Signal;
import com.sodius.mdw.metamodel.uml21.State;
import com.sodius.mdw.metamodel.uml21.StructuredActivityNode;
import com.sodius.mdw.metamodel.uml21.Transition;

public class UnitsTreeImportContentProvider implements ITreeContentProvider {

	private final UnitsSelectionControl control;
	
	UnitsTreeImportContentProvider(UnitsSelectionControl control) {
		super();
		this.control = control;
	}
	
	private boolean isHiddenAlwaysSelectedUnit(Namespace unit)
	{
		//standard Models and Profiles
		if(unit instanceof com.sodius.mdw.metamodel.uml21.Package && unit.getAppliedStereotype("Ecore::EPackage")!=null)
			return true;
		//UMLPrimitiveTypes
		if(unit instanceof com.sodius.mdw.metamodel.uml21.Model && "UMLPrimitiveTypes".equals(unit.getQualifiedName()))
			return true;
		//Rhapsody Standard Model
		if(unit instanceof com.sodius.mdw.metamodel.uml21.Model && "RhapsodyStandardModel".equals(unit.getQualifiedName()))
			return true;
		
		if(unit.getOwner()==null && isHiddenUnitsSelectedWithParents(unit))
			return true;
		return false;
	}
	
	private boolean isHiddenUnitsSelectedWithParents(Namespace unit)
	{
		if(unit instanceof BehavioralFeature)
			return true;
		if(unit instanceof Association)
			return true;
		if(unit instanceof InformationItem)
			return true;
		if(unit instanceof Signal)
			return true;
		if(unit instanceof OpaqueBehavior)
			return true;
		if(unit instanceof InteractionOperand)
			return true;
		if(unit instanceof State)
			return true;
		if(unit instanceof Region)
			return true;
		if(unit instanceof StructuredActivityNode)
			return true;
		if(unit instanceof Transition)
			return true;
		return false;
	}
	
	public Object[] getChildren(Object parent) {
		return getChildren(parent, true);
	}
	private Object[] getChildren(Object parent, boolean registerHidden) {
		if (parent instanceof Model){
			Model model = (Model)parent;
			return filterChildren(rejectContainedElements(model.<Namespace>getInstances("Namespace", true)), parent, registerHidden);
		}
		else if (parent instanceof Namespace) {
			Namespace namespace = (Namespace)parent;
			return filterChildren(namespace.getOwnedElements().getInstances("Namespace", true), parent, registerHidden);
		}
		return null;
	}
	
	private List<Namespace> rejectContainedElements(List<Namespace> elements) //@See RTC 140913
	{
		List<Namespace> notOwnedElements = new ArrayList<Namespace>();
		for (Namespace element : elements) {
			if(element.eContainer()==null)
				notOwnedElements.add(element);
		}
		return notOwnedElements;
	}
	
	private Object[] filterChildren(List<? extends Element> children, Object parent, boolean registerHidden)
	{
		List<Namespace> returnList = new ArrayList<Namespace>();
		for (Element child : children) {
			if(child instanceof Namespace){
				Namespace childNS = (Namespace)child;
				if(isHiddenAlwaysSelectedUnit(childNS))
				{
					if(registerHidden)
						control.addHiddenUnitsAlwaysSelected(childNS);
				}
				else if(isHiddenUnitsSelectedWithParents(childNS))
				{
					if(registerHidden)
						control.addHiddenUnitsSelectedWithParents(parent, childNS);
				}
				else
					returnList.add(childNS);
			}
		}
		
		return returnList.toArray();
	}

	public Object getParent(Object element) {
		
		return null;
	}

	public boolean hasChildren(Object element) {
		return (element instanceof Namespace && (getChildren(element,false).length>0));
	}

	public Object[] getElements(Object inputElement) {
		if (inputElement !=null && inputElement instanceof Model) {
			Model model = ((Model) inputElement);
			return getChildren(model);
		}
		return null;
	}

	public void dispose() {
	}

	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		// TODO Auto-generated method stub

	}

}
