package com.sodius.rhapsody.toolkit.ui;

import java.io.File;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.swt.graphics.Image;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.metamodel.uml21.Activity;
import com.sodius.mdw.metamodel.uml21.Actor;
import com.sodius.mdw.metamodel.uml21.Artifact;
import com.sodius.mdw.metamodel.uml21.Association;
import com.sodius.mdw.metamodel.uml21.AssociationClass;
import com.sodius.mdw.metamodel.uml21.Collaboration;
import com.sodius.mdw.metamodel.uml21.CommunicationPath;
import com.sodius.mdw.metamodel.uml21.Component;
import com.sodius.mdw.metamodel.uml21.ConditionalNode;
import com.sodius.mdw.metamodel.uml21.DataType;
import com.sodius.mdw.metamodel.uml21.DeploymentSpecification;
import com.sodius.mdw.metamodel.uml21.Device;
import com.sodius.mdw.metamodel.uml21.Element;
import com.sodius.mdw.metamodel.uml21.Enumeration;
import com.sodius.mdw.metamodel.uml21.ExecutionEnvironment;
import com.sodius.mdw.metamodel.uml21.ExpansionRegion;
import com.sodius.mdw.metamodel.uml21.Extension;
import com.sodius.mdw.metamodel.uml21.FinalState;
import com.sodius.mdw.metamodel.uml21.FunctionBehavior;
import com.sodius.mdw.metamodel.uml21.InformationItem;
import com.sodius.mdw.metamodel.uml21.Interaction;
import com.sodius.mdw.metamodel.uml21.InteractionOperand;
import com.sodius.mdw.metamodel.uml21.Interface;
import com.sodius.mdw.metamodel.uml21.LoopNode;
import com.sodius.mdw.metamodel.uml21.Model;
import com.sodius.mdw.metamodel.uml21.Node;
import com.sodius.mdw.metamodel.uml21.OpaqueBehavior;
import com.sodius.mdw.metamodel.uml21.Operation;
import com.sodius.mdw.metamodel.uml21.Package;
import com.sodius.mdw.metamodel.uml21.PrimitiveType;
import com.sodius.mdw.metamodel.uml21.Profile;
import com.sodius.mdw.metamodel.uml21.ProtocolStateMachine;
import com.sodius.mdw.metamodel.uml21.ProtocolTransition;
import com.sodius.mdw.metamodel.uml21.Reception;
import com.sodius.mdw.metamodel.uml21.Region;
import com.sodius.mdw.metamodel.uml21.SequenceNode;
import com.sodius.mdw.metamodel.uml21.Signal;
import com.sodius.mdw.metamodel.uml21.State;
import com.sodius.mdw.metamodel.uml21.StateMachine;
import com.sodius.mdw.metamodel.uml21.Stereotype;
import com.sodius.mdw.metamodel.uml21.StructuredActivityNode;
import com.sodius.mdw.metamodel.uml21.Transition;
import com.sodius.rhapsody.toolkit.util.RhapsodyUnitHandler;
import com.sodius.rhapsody.toolkit.util.thread.RhapsodyAPIExec;
import com.telelogic.rhapsody.core.IRPActor;
import com.telelogic.rhapsody.core.IRPAssociationClass;
import com.telelogic.rhapsody.core.IRPClass;
import com.telelogic.rhapsody.core.IRPComponent;
import com.telelogic.rhapsody.core.IRPControlledFile;
import com.telelogic.rhapsody.core.IRPEvent;
import com.telelogic.rhapsody.core.IRPEventReception;
import com.telelogic.rhapsody.core.IRPFile;
import com.telelogic.rhapsody.core.IRPFlowItem;
import com.telelogic.rhapsody.core.IRPFlowchart;
import com.telelogic.rhapsody.core.IRPInstance;
import com.telelogic.rhapsody.core.IRPModule;
import com.telelogic.rhapsody.core.IRPNode;
import com.telelogic.rhapsody.core.IRPOperation;
import com.telelogic.rhapsody.core.IRPPackage;
import com.telelogic.rhapsody.core.IRPPort;
import com.telelogic.rhapsody.core.IRPProfile;
import com.telelogic.rhapsody.core.IRPProject;
import com.telelogic.rhapsody.core.IRPRelation;
import com.telelogic.rhapsody.core.IRPStatechart;
import com.telelogic.rhapsody.core.IRPStereotype;
import com.telelogic.rhapsody.core.IRPSysMLPort;
import com.telelogic.rhapsody.core.IRPType;
import com.telelogic.rhapsody.core.IRPUnit;
import com.telelogic.rhapsody.core.IRPUseCase;

public class UnitsTreeLabelProvider implements ILabelProvider {

	private ToolkitUIManager manager;

	public UnitsTreeLabelProvider(ToolkitUIManager manager) {
		this.manager = manager;
	}

	
	public Image getImage(Element obj)
	{
		if (obj instanceof DeploymentSpecification) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_FILE);
		} else if (obj instanceof Artifact) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_FILE);
		} else if (obj instanceof AssociationClass) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_ACTOR);
		} else if (obj instanceof Activity) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_FLOWCHART);
		} else if (obj instanceof Interaction) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_SEQUENCEDIAGRAM);
		} else if (obj instanceof StateMachine) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_STATECHART);
		} else if (obj instanceof Component) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_COMPONENT);
		} else if (obj instanceof Node) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_NODE);
		} else if (obj instanceof Stereotype) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_STEREOTYPE);
		} else if (obj instanceof Collaboration) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_COLLABORATIONDIAGRAM);
		} else if (obj instanceof DataType) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_TYPE);
		} else if (obj instanceof Interface) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_CLASS);
		} else if (obj instanceof Signal) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_EVENT);
		} else if (obj instanceof Model) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_PROJECT);
		} else if (obj instanceof Profile) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_PROFILE);
		} else if (obj instanceof Package) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_PACKAGE);
		} else {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_CLASS);
		}
	}
	
	public Image getImage(RhapsodyUnitHandler obj)
	{
		if (obj.getIRPInterface() instanceof IRPActor) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_ACTOR);
		} else if (obj.getIRPInterface() instanceof IRPAssociationClass) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_ASSOCIATIONCLASS);
		} else if (obj.getIRPInterface() instanceof IRPFlowchart) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_FLOWCHART);
		} else if (obj.getIRPInterface() instanceof IRPStatechart) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_STATECHART);
		} else if (obj.getIRPInterface() instanceof IRPClass) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_CLASS);
		} else if (obj.getIRPInterface() instanceof IRPComponent) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_COMPONENT);
		} else if (obj.getIRPInterface() instanceof IRPControlledFile) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_CONTROLLEDFILE);
		} else if (obj.getIRPInterface() instanceof IRPEvent) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_EVENT);
		} else if (obj.getIRPInterface() instanceof IRPEventReception) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_EVENTRECEPTION);
		} else if (obj.getIRPInterface() instanceof IRPFile) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_FILE);
		} else if (obj.getIRPInterface() instanceof IRPFlowItem) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_FLOWITEM);
		} else if (obj.getIRPInterface() instanceof IRPModule) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_MODULE);
		} else if (obj.getIRPInterface() instanceof IRPPort) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_PORT);
		} else if (obj.getIRPInterface() instanceof IRPSysMLPort) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_SYSMLPORT);
		} else if (obj.getIRPInterface() instanceof IRPInstance) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_INSTANCE);
		} else if (obj.getIRPInterface() instanceof IRPNode) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_NODE);
		} else if (obj.getIRPInterface() instanceof IRPOperation) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_OPERATION);
		} else if (obj.getIRPInterface() instanceof IRPProfile) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_PROFILE);
		} else if (obj.getIRPInterface() instanceof IRPProject) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_PROJECT);
		} else if (obj.getIRPInterface() instanceof IRPPackage) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_PACKAGE);
		} else if (obj.getIRPInterface() instanceof IRPRelation) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_RELATION);
		} else if (obj.getIRPInterface() instanceof IRPStereotype) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_STEREOTYPE);
		} else if (obj.getIRPInterface() instanceof IRPType) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_TYPE);
		} else if (obj.getIRPInterface() instanceof IRPUseCase) {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_USECASE);
		}
	
		else {
			return getRHPImageKeyRegistry(ToolkitUIManager.RHP_CLASS);
		}
	}
	
	public Image getImage(Object obj) {
		if(obj instanceof Element)
			return getImage((Element)obj);
		if(obj instanceof RhapsodyUnitHandler)
			return getImage((RhapsodyUnitHandler)obj);
		
		return getRHPImageKeyRegistry(ToolkitUIManager.RHP_CLASS);
	}

	private Image getRHPImageKeyRegistry(String name) {
		return this.manager.getImageRegistry().get(ToolkitUIManager.RHP_FOLDER + "/" + name);
	}

	public String getText(final Object obj) {
		if (obj instanceof RhapsodyUnitHandler) {
			return ((RhapsodyUnitHandler)obj).getName();
		} else
			return obj.toString();
	}

	public void addListener(ILabelProviderListener listener) {
		// TODO Auto-generated method stub

	}

	public void dispose() {
		// TODO Auto-generated method stub

	}

	public boolean isLabelProperty(Object element, String property) {
		// TODO Auto-generated method stub
		return false;
	}

	public void removeListener(ILabelProviderListener listener) {
		// TODO Auto-generated method stub

	}
}
