package com.sodius.rhapsody.toolkit.util;

import java.util.ArrayList;
import java.util.List;

import com.sodius.rhapsody.toolkit.engine.XMIEngine;
import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.IRPProject;
import com.telelogic.rhapsody.core.IRPUnit;

public class RhapsodyUnitHandler {

	private final String GUID;
	private final String name;
	private final IRPModelElement IRPInterface;
//	private final RhapsodyItemKind kind;
//	public enum RhapsodyItemKind{
//		IRPProject,
//		IRPModelElement
//	};
	
	public RhapsodyUnitHandler(final IRPModelElement unit)
	{
		GUID = unit.getGUID();
		name = unit.getName();
		IRPInterface = unit;
	}
	
	public String getGUID()
	{
		return GUID;
	}
	
	public String getName()
	{
		return name;
	}
	
	public IRPModelElement getIRPInterface()
	{
		return IRPInterface;
	}
	
	public static List<Object> createList(String[] namespaceList, IRPProject project)
	{
		List<Object> result = new ArrayList<Object>();
		for (String namespace : namespaceList) {
			try{
				int indexOfMetaclassSeparator = namespace.indexOf("#");
				String fullPath = namespace.substring(0, indexOfMetaclassSeparator);
				String metaclass = namespace.substring(indexOfMetaclassSeparator+1);
				IRPUnit unit = (IRPUnit)project.findElementsByFullName(fullPath, metaclass);
				result.add(new RhapsodyUnitHandler(unit));
			}catch (Throwable e) {
				throw new IllegalArgumentException("Unit selection not well defined.",e);
			}
		}
		
		return result;
	}
	
	@Override
	public int hashCode() {
		return (""+GUID).hashCode();
	}
	
	@Override
	public boolean equals(Object obj) {
		if(obj instanceof RhapsodyUnitHandler)
		{
			return hashCode()==obj.hashCode();
		}
		return false;
	}
}
