package com.sodius.rhapsody.toolkit.util.thread;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.OperationCanceledException;

public class RhapsodyAPIExec {
	
	private static RhapsodyAPIThread RHAPSODY_DEFAULT_DAEMON = null;
	public static Runnable FINISH_THREAD = new Runnable() {
		
		public void run() {
			RHAPSODY_DEFAULT_DAEMON.shouldQuit=true;
			RHAPSODY_DEFAULT_DAEMON=null;
		}
	};
	
	private static class RhapsodyAPIThread extends Thread {
		protected boolean shouldQuit=false;
		private boolean isRunning=false;
		private Throwable lastError;
		private Runnable currentRunnable=null;
		//private MegaConnection context;
		private Object lock = new Object();
		private boolean started = false;
		
		RhapsodyAPIThread(){
			super("MDWorkbench Rhapsody Connection");
			setDaemon(true); // The Java Virtual Machine exits 
							 // when the only threads running are all daemon threads. 
		}
		
		public void run() {
			while(!shouldQuit){
				try {
					started = true;
					if(currentRunnable!=null){
						currentRunnable.run();
						currentRunnable = null;
						isRunning = false;
						synchronized (lock) {
							lock.notifyAll();
						}
					}
					try {
						Thread.sleep(50);
					} catch (InterruptedException e) {
						// nothing
					}
				}catch (Throwable e) { // never stop the tread on error
					lastError = e;
					currentRunnable = null;
					isRunning = false;
					synchronized (lock) {
						lock.notifyAll();
					}
				}
			}
		}
		
		private void synchExec(Runnable runnable) throws CoreException {
				if(RHAPSODY_DEFAULT_DAEMON!=null && RHAPSODY_DEFAULT_DAEMON.isAlive())
					RHAPSODY_DEFAULT_DAEMON.interrupt();
				lastError = null;
				isRunning = true;
				if(currentRunnable!=null)
					throw new CoreException("Concurrent Rhapsody access, previous runnable not stopped");
				currentRunnable = runnable;
				if(isRunning){
					synchronized (lock) {
						try {
							lock.wait();
						} catch (InterruptedException e) {
							// nothing
						} catch (Throwable e) {
							throw new CoreException(e);
						}
					}
				}
				if(lastError!=null){
					if(lastError instanceof OperationCanceledException){
						throw (OperationCanceledException)lastError;
					}else if(lastError instanceof CoreException){
						throw (CoreException)lastError ;
					}else{
						throw new CoreException(lastError);
					}
				}
			
		}
	}
	
	public static boolean NO_THREAD = false;
	
	public static void synchExec(Runnable runnable) throws CoreException {
		if(isInRhapsodyThread())
		{
			runnable.run();
		}
		else if (NO_THREAD){
			runnable.run();
		}else{
			if(RHAPSODY_DEFAULT_DAEMON==null){
				RHAPSODY_DEFAULT_DAEMON = new RhapsodyAPIThread();
				RHAPSODY_DEFAULT_DAEMON.start();
				while (!RHAPSODY_DEFAULT_DAEMON.started)  // waiting the thread to be started
				{
					try {//@See RTC 142092 
						Thread.sleep(10);
					} catch (InterruptedException e) {
						// nothing
					}
				}
//				Runtime.getRuntime().addShutdownHook(new Thread(){
//					public void run() {
//						if(RHAPSODY_DEFAULT_DAEMON.context!=null)
//							RHAPSODY_DEFAULT_DAEMON.context.releaseAll();
//					}
//				});
			}
		
			RHAPSODY_DEFAULT_DAEMON.synchExec(runnable);
		}
	}
	
	public static boolean isInRhapsodyThread()
	{
		return (Thread.currentThread() instanceof RhapsodyAPIThread);
	}

}
