@rem ***************************************************************************
@rem Licensed Materials - Property of IBM
@rem (c) Copyright IBM Corporation 2018, 2020. All Rights Reserved.
@rem
@rem Note to U.S. Government Users Restricted Rights: 
@rem Use, duplication or disclosure restricted by GSA ADP Schedule 
@rem Contract with IBM Corp.
@rem ***************************************************************************
@echo off
setlocal EnableDelayedExpansion

if [%1]==[] (
	echo This batch file should not be executed directly. use setup.bat
	goto finish
)

set VER1=%1
set VER2=%2
set ARCH=%3
set IFIX_ARCH=%4
for /f "tokens=4,* delims= " %%a in ("%*") do set INSTALL_ARGS=%%b

echo Checking preconditions prior the install
echo.

rem checking write permission
set RegQry=HKLM\Hardware\Description\System\CentralProcessor\0
%SystemRoot%\System32\REG.exe Query %RegQry% | %SystemRoot%\System32\FindSTR /C:"x86" >nul & IF  ERRORLEVEL 1 (
	set OS_ARCH=x64
) else (
	set OS_ARCH=x86
)
echo OS_ARCH Arhitecture is %OS_ARCH%

if %OS_ARCH%==x64 (
	set REGKEY="HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion"
	) else (
	set REGKEY="HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion"
)

rem get registry values for rhapsody root path
set REGKEY="HKEY_LOCAL_MACHINE\SOFTWARE\Telelogic\Rhapsody\%VER1%"
if "%ARCH%"=="32bit" (
	if %OS_ARCH%==x64 (
		set REGKEY="HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Telelogic\Rhapsody\%VER1%"
	)
)
set RHPROOT=InstallationDirectory

FOR /F "usebackq tokens=2*" %%A IN (`%SystemRoot%\System32\reg QUERY %REGKEY% /v %RHPROOT%`) DO (
    set RHPPath=%%B
)

if defined RHPPath (
    echo Found Rhapsody root path:  "%RHPPath%"
	echo Continue installation...
) else (
    echo %REGKEY%\%RHPROOT% was not found in the registry.
	echo installation will not continue. Exiting...
	goto finish
)

rem get registry values for rhapsody share path
echo.
set RHPSHARE=UserDirectory

FOR /F "usebackq tokens=2*" %%A IN (`%SystemRoot%\System32\reg QUERY %REGKEY% /v %RHPSHARE%`) DO (
    set RHPUserDataRootPath=%%B
)

if defined RHPUserDataRootPath (
    echo Found Rhapsody user data root path: "%RHPUserDataRootPath%"
	echo Continue installation...    
) else (
    echo %REGKEY%\%RHPSHARE% was not found in the registry.
	echo installation will not continue. Exiting...
	goto finish
)

rem check if root folder exists
echo.
if exist "%RHPPath%" (
	rem remove trailing backslash and adding double quotes
	IF %RHPPath:~-1%==\ SET RHPPath="%RHPPath:~0,-1%"
	echo "%RHPPath%" exists. Continue...
) else (
    echo "%RHPPath%" directory not found.
	echo installation will not continue. Exiting...
	goto finish
)

rem check if share folder exists
if exist "%RHPUserDataRootPath%" (
	rem remove trailing backslash and adding double quotes
	IF %RHPUserDataRootPath:~-1%==\ SET RHPUserDataRootPath="%RHPUserDataRootPath:~0,-1%"
	echo "%RHPUserDataRootPath%" exists. Continue...
) else (
    echo "%RHPUserDataRootPath%" directory not found.
	echo installation will not continue. Exiting...
	goto finish
)

rem Now both RHPPath and RHPUserDataRootPath has double quotes and backslash at the end. e.g. "C:\My Test\Rhpsody\"

rem echo ****************** Start Rhapsody iFix pre_install script ********************
rem call iFixContent\pre_install.bat
rem echo ****************** Finished Rhapsody iFix pre_install script *****************
setlocal DisableDelayedExpansion

set RHPBackupFolder=%TEMP%\RhapsodyiFixTemp%VER1%
echo.
echo.
echo ****************** Backing up some files to %RHPBackupFolder% ********************
%SystemRoot%\System32\Robocopy %RHPUserDataRootPath% %RHPBackupFolder%\RhapsodyShare com.ibm.rational.rhapsody.rmm.svg_6.0.6*.jar   _6.0.6*.jar /S  /MOV /R:5 /W:3 /NDL
rem iFix03 files have the same name and replaced no need to delete
rem %SystemRoot%\System32\Robocopy %RHPPath% %RHPBackupFolder%\RhapsodyRoot com.sodius.mdw.UML21toRhapsody_8.4*.jar com.sodius.toolkit.rhapsody_8.4*.jar /S  /MOV /R:5 /W:3 /NDL
%SystemRoot%\System32\Robocopy %RHPPath%\RTCIntegration %RHPBackupFolder%\RTCIntegration /S /MOV /R:5 /W:3 /NDL
echo.
echo ****************** Finished Backing up files ********************
echo.


echo.
echo.
echo ****************** Start Rhapsody iFix Installation ********************
iFixContent\Rhapsody%VER2%iFix_%IFIX_ARCH%.exe %INSTALL_ARGS%
if not errorlevel 0 goto handleerror
echo iFixContent\Rhapsody%VER2%iFix_%IFIX_ARCH%.exe %INSTALL_ARGS% installation finished successfully. Continue... 
echo.

rem copy files to root and share
echo.
echo ****************** Start Rhapsody iFix Files copy ********************
echo.
echo copy root files and folders to %RHPPath%
%SystemRoot%\System32\Robocopy iFixContent\RhapsodyRoot %RHPPath% /S /E /R:5 /W:3
echo.
echo copy share files and folders to %RHPUserDataRootPath%\Share
echo.
%SystemRoot%\System32\Robocopy iFixContent\RhapsodyShare %RHPUserDataRootPath%\Share /S /E /R:5 /W:3
echo.
echo ****************** Finished Rhapsody iFix Files copy ********************
echo.

rem check if XMI folder exists
set XMIToolkit=%RHPUserDataRootPath%\Sodius\XMI_Toolkit
if exist %XMIToolkit% (
	rem copy XMIToolkit files
	echo.
	echo ****************** Start XMIToolkit iFix Files copy ********************
	echo.
	echo copy XMI files and folders to %XMIToolkit%
	%SystemRoot%\System32\Robocopy iFixContent\XMI_Toolkit %XMIToolkit% /S /E /R:5 /W:3
	echo.
	echo ****************** Finished XMIToolkit iFix Files copy ********************
	echo.
)

rem run post install scripts
echo ****************** Start Rhapsody iFix post_install script ********************
echo.
call iFixContent\post_install.bat %ARCH% %VER1% 
echo ****************** Finished Rhapsody iFix post_install script ********************
goto finish

:handleerror
echo Rhapsody%VER2%iFix_%IFIX_ARCH%.exe installation failed. Exiting...
echo.
echo ****************** Restoring files from backup - due to error in installation ********************
%SystemRoot%\System32\Robocopy %RHPBackupFolder%\RhapsodyShare %RHPUserDataRootPath% /S /MOVE /R:5 /W:3 /NDL /XX
%SystemRoot%\System32\Robocopy %RHPBackupFolder%\RhapsodyRoot %RHPPath% /S /MOVE /R:5 /W:3 /NDL /XX
%SystemRoot%\System32\Robocopy %RHPBackupFolder%\RTCIntegration %RHPPath%\RTCIntegration /S /MOVE /R:5 /W:3 /NDL /XX
echo.

:finish
rmdir /S /Q %RHPBackupFolder%
ENDLOCAL 
echo.
echo ************ Finished Rhapsody iFix Installation *****************
echo.
